/* demo_xy_problems_1.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/*
* problems 1 .. 10
*/

#include <stdio.h>

#include "lbmn_mesh_xy_D.h"
#include "demo_xy_problems_1.h"


int
lbmn_demo_xy_problem_1_2_plates_200(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 200;
    unsigned int problem_size_col = 200;
    double problem_x_start = 0.0;
    double problem_x_stop  = 199.0;
    double problem_y_start = 0.0;
    double problem_y_stop  = 199.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 200;
	    i_row_start   = 1;
	    i_row_stop    = 200;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
	
	    /* left plate */
	
	    i_col_start   = 81;
        i_col_stop    = 90;
	    i_row_start   = 51;
	    i_row_stop    = 150;
        boundary      = +1;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("plate 1 allocated\n");

	    /* right plate */
	
        i_col_start = 111;
	    i_col_stop  = 120;
	    i_row_start = 51;
	    i_row_stop  = 150;
        boundary    = +2;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    printf("plate 2 allocated\n");

    }/* memory allocated */
    return 0;
}/* lbmn_demo_xy_problem_1__2_plates_200 */


/*============================================================================*/

int
lbmn_demo_xy_problem_2_2_plates_400(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 400;
    unsigned int problem_size_col = 400;
    double problem_x_start = 0.0;
    double problem_x_stop  = 399.0;
    double problem_y_start = 0.0;
    double problem_y_stop  = 399.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 400;
	    i_row_start   = 1;
	    i_row_stop    = 400;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
	
	    /* left plate */
	
	    i_col_start   = 181;
        i_col_stop    = 190;
	    i_row_start   = 101;
	    i_row_stop    = 300;
        boundary      = +1;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("plate 1 allocated\n");

	    /* right plate */
	
        i_col_start = 211;
	    i_col_stop  = 220;
	    i_row_start = 101;
	    i_row_stop  = 300;
        boundary    = +2;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    printf("plate 2 allocated\n");

    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_2__2_plates_400 */

/*================================================*/

int
lbmn_demo_xy_problem_3_flat_diode_201(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 201;
    unsigned int problem_size_col = 201;
    double problem_x_start = -100.0;
    double problem_x_stop  = +100.0;
    double problem_y_start = -100.0;
    double problem_y_stop  = +100.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 201;
	    i_row_start   = 1;
	    i_row_stop    = 201;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
	
        /* cathode consists of parts A, B and C*/
        
	    /* A */
	
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 201;
        boundary      = +1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* B */
	
        i_col_start = 11;
	    i_col_stop  = 191;
	    i_row_start = 192;
	    i_row_stop  = 201;
        boundary    = +1;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

	
	    /* C */
	
        i_col_start = 11;
	    i_col_stop  = 191;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		
	    /* anode (D) */
	
        i_col_start = 192;
	    i_col_stop  = 201;
	    i_row_start = 1;
	    i_row_stop  = 201;
        boundary    = +2;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_3_flat_diode_201 */


/*========================================================*/

int
lbmn_demo_xy_problem_4_S_rectangle_500_300(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 500;
    unsigned int problem_size_col = 300;
    double problem_x_start = +0.0;
    double problem_x_stop  = +299;
    double problem_y_start = +0.0;
    double problem_y_stop  = +499.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 500;
	    i_row_start   = 1;
	    i_row_stop    = 300;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set inner electrode points  *
        *=============================*/
	
        /* cathode consists of parts A, B, C, D, E, F*/
        
	    /* A */
	
	    i_col_start   = 101;
        i_col_stop    = 110;
	    i_row_start   = 101;
	    i_row_stop    = 200;
        boundary      = +1;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* B */
	
        i_col_start = 101;
	    i_col_stop  = 200;
	    i_row_start = 201;
	    i_row_stop  = 210;
        boundary    = +1;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

	
	    /* C */
	
        i_col_start = 201;
	    i_col_stop  = 210;
	    i_row_start = 201;
	    i_row_stop  = 310;
        boundary    = +1;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		
	    /* D */
	
        i_col_start = 101;
	    i_col_stop  = 200;
	    i_row_start = 301;
	    i_row_stop  = 310;
        boundary    = +1;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

 	    /* E */
	
        i_col_start = 101;
	    i_col_stop  = 110;
	    i_row_start = 311;
	    i_row_stop  = 400;
        boundary    = +1;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* F */
	
        i_col_start = 101;
	    i_col_stop  = 210;
	    i_row_start = 401;
	    i_row_stop  = 410;
        boundary    = +1;
        voltage     = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);




        /*===== border set to zero V  ====*/
        /* X, Y, Z, T */
	
        /* X */

        i_col_start = 1;
	    i_col_stop  = 10;
	    i_row_start = 1;
	    i_row_stop  = 500;
        boundary    = +2;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* Y */

        i_col_start = 291;
	    i_col_stop  = 300;
	    i_row_start = 1;
	    i_row_stop  = 500;
        boundary    = +2;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    
        /* Z */

        i_col_start = 11;
	    i_col_stop  = 290;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +2;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* T */

        i_col_start = 11;
	    i_col_stop  = 290;
	    i_row_start = 491;
	    i_row_stop  = 500;
        boundary    = +2;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_4_S_rectangle_500_300 */


/*========================================================*/

int
lbmn_demo_xy_problem_5_electrolytic_tank_601_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 601;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -300.0;
    double problem_y_stop  = +300.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	unsigned int center_row = 0.0;
	unsigned int center_col = 0.0;
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double point_x;
	double point_y;
	
    int ret_val = 0;

    /* instructions */

	
	/* central node of coordinate system (0,0) */
	center_row = 301;
	center_col = 201; 
	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set frame electrode points  *
        *=============================*/
	
        /* cathode consists of parts A, B, C, D */
        
	    /* A left*/
	
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = +1;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* B right */
	
        i_col_start = 392;
	    i_col_stop  = 401;
	    i_row_start = 1;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

	
	    /* C down */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		
	    /* D up */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 592;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


		/* set round electrodes X, Y */


 	    /* X up*/
	
		electrode_radius = 50.0;
		
        boundary    = +2;
        voltage     = -1.0;


		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row + 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* Y down */
	
		electrode_radius = 50.0;
		
        boundary    = +3;
        voltage     = +1.0;

		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row - 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_5_electrolytic_tank_601_401 */

/*=========================================================================*/

int
lbmn_demo_xy_problem_6_dualsegment_magnetron_smile_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double point_x;
	double point_y;
	
    int ret_val = 0;

    /* instructions */

	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points  *
        *=============================*/
	
        /* Duants A, B, cathode C*/
        


 	    /* A - lower duant*/
	
		electrode_radius = 200.0;
		
        boundary    = +1;
        voltage     = -1.0;


		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius &&
					point_y < 0.0)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* B - upper duant */
	
		electrode_radius = 200.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius &&
				  point_y > 0.0)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
		
		/* C - cathode */
	
		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = +0.5;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius )
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_6_dualsegment_magnetron_smile_401_401 */


/*======================================================================*/
int
lbmn_demo_xy_problem_7_dualsegment_magnetron_Paszkowski_p62_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double point_x;
	double point_y;
	
    int ret_val = 0;

    /* instructions */

	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +200.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points  *
        *=============================*/
	
        /* Duants A, B, cathode C*/
        


 	    /* A - lower duant*/
	
		electrode_radius = 200.0;
		
        boundary    = +1;
        voltage     = +450.0;


		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius &&
					point_y < -10.0)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* B - upper duant */
	
		electrode_radius = 200.0;
		
        boundary    = +2;
        voltage     = +550.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius &&
				  point_y > 10.0)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
		
		/* C - cathode */
	
		electrode_radius = 10.0;
		
        boundary    = +3;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius )
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_7_dualsegment_magnetron_Paszkowski_p62_401_401 */


/*======================================================================*/

int
lbmn_demo_xy_problem_8_flat_diaphragm_Paszkowski_p62_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;


    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */



	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        /*============================*
        *   set electrode points      *
        *=============================*/
	

        /* left side */
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* left bottom */
        i_col_start   = 11;
        i_col_stop    = 190;
	    i_row_start   = 1;
	    i_row_stop    = 10;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);



        /* left upper */
        i_col_start   = 11;
        i_col_stop    = 190;
	    i_row_start   = 392;
	    i_row_stop    = 401;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


        

        /* right side */
	    i_col_start   = 392;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* right bottom */
        i_col_start   = 212;
        i_col_stop    = 391;
	    i_row_start   = 1;
	    i_row_stop    = 10;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);



        /* right upper */
        i_col_start   = 212;
        i_col_stop    = 391;
	    i_row_start   = 392;
	    i_row_stop    = 401;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


        /* diaphragm bottom */
        i_col_start   = 191;
        i_col_stop    = 211;
	    i_row_start   = 1;
	    i_row_stop    = 151;
        boundary      = 3;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* diaphragm upper */
        i_col_start   = 191;
        i_col_stop    = 211;
	    i_row_start   = 251;
	    i_row_stop    = 401;
        boundary      = 3;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_8_flat_diaphragm_Paszkowski_p62_401_401 */

/*=========================================================*/

int
lbmn_demo_xy_problem_9_2_inequal_plates_200(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 200;
    unsigned int problem_size_col = 200;
    double problem_x_start = 0.0;
    double problem_x_stop  = 199.0;
    double problem_y_start = 0.0;
    double problem_y_stop  = 199.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 200;
	    i_row_start   = 1;
	    i_row_stop    = 200;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
	
	    /* left plate */
	
	    i_col_start   = 81;
        i_col_stop    = 90;
	    i_row_start   = 51;
	    i_row_stop    = 120;/* inequal to right plate */
        boundary      = +1;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("plate 1 allocated\n");

	    /* right plate */
	
        i_col_start = 111;
	    i_col_stop  = 120;
	    i_row_start = 51;
	    i_row_stop  = 150;/* inequal */
        boundary    = +2;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    printf("plate 2 allocated\n");

    }/* memory allocated */
    return 0;
}/* lbmn_demo_xy_problem_9_2_inequal_plates_200 */

/*=====================================================*/

int
lbmn_demo_xy_problem_10_modified_flat_diaphragm_Paszkowski_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;


    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */



	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        /*============================*
        *   set electrode points      *
        *=============================*/
	

        /* left side */
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* left bottom */
        i_col_start   = 11;
        i_col_stop    = 190;
	    i_row_start   = 1;
	    i_row_stop    = 10;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);



        /* left upper */
        i_col_start   = 11;
        i_col_stop    = 190;
	    i_row_start   = 392;
	    i_row_stop    = 401;
        boundary      = 1;
        voltage       = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


        

        /* right side */
	    i_col_start   = 392;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* right bottom */
        i_col_start   = 212;
        i_col_stop    = 391;
	    i_row_start   = 1;
	    i_row_stop    = 10;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);



        /* right upper */
        i_col_start   = 212;
        i_col_stop    = 391;
	    i_row_start   = 392;
	    i_row_stop    = 401;
        boundary      = 2;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


        /* diaphragm bottom */
        i_col_start   = 191;
        i_col_stop    = 211;
	    i_row_start   = 1;
	    i_row_stop    = 151;
        boundary      = 3;
        voltage       = +0.3;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

        /* diaphragm upper */
        i_col_start   = 191;
        i_col_stop    = 211;
	    i_row_start   = 251;
	    i_row_stop    = 401;
        boundary      = 4;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_10_modified_flat_diaphragm_Paszkowski_401_401 */

/* ==== end of file demo_xy_problems_1.c ==== */
