/* demo_xy_problems_2.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 
  /*
 * problems 11 .. 20
 */

#include <stdio.h>

#include "lbmn_mesh_xy_D.h"
#include "demo_xy_problems_2.h"

int
lbmn_demo_xy_problem_11_electrolytic_tank_inequal_601_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 601;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -300.0;
    double problem_y_stop  = +300.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	unsigned int center_row = 0.0;
	unsigned int center_col = 0.0;
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double point_x;
	double point_y;
	
    int ret_val = 0;

    /* instructions */

	
	/* central node of coordinate system (0,0) */
	center_row = 301;
	center_col = 201; 
	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set frame electrode points  *
        *=============================*/
	
        /* cathode consists of parts A, B, C, D */
        
	    /* A left*/
	
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = +1;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* B right */
	
        i_col_start = 392;
	    i_col_stop  = 401;
	    i_row_start = 1;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

	
	    /* C down */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		
	    /* D up */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 592;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


		/* set round electrodes X, Y */


 	    /* X up*/
	
		electrode_radius = 50.0;
		
        boundary    = +2;
        voltage     = -1.0;


		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row + 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* Y down */
	
		electrode_radius = 100.0;
		
        boundary    = +3;
        voltage     = +1.0;

		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row - 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_11_electrolytic_tank_inequal_601_401 */

/*=================================================================*/

int
lbmn_demo_xy_problem_12_electrolytic_tank_equal_pots_601_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 601;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -300.0;
    double problem_y_stop  = +300.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	unsigned int center_row = 0.0;
	unsigned int center_col = 0.0;
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	double point_x;
	double point_y;
	
    int ret_val = 0;

    /* instructions */

	
	/* central node of coordinate system (0,0) */
	center_row = 301;
	center_col = 201; 
	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set frame electrode points  *
        *=============================*/
	
        /* cathode consists of parts A, B, C, D */
        
	    /* A left*/
	
	    i_col_start   = 1;
        i_col_stop    = 10;
	    i_row_start   = 1;
	    i_row_stop    = 601;
        boundary      = +1;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


	    /* B right */
	
        i_col_start = 392;
	    i_col_stop  = 401;
	    i_row_start = 1;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

	
	    /* C down */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 1;
	    i_row_stop  = 10;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

		
	    /* D up */
	
        i_col_start = 11;
	    i_col_stop  = 391;
	    i_row_start = 592;
	    i_row_stop  = 601;
        boundary    = +1;
        voltage     = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);


		/* set round electrodes X, Y */


 	    /* X up*/
	
		electrode_radius = 50.0;
		
        boundary    = +2;
        voltage     = +1.0;


		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row + 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* Y down */
	
		electrode_radius = 100.0;
		
        boundary    = +3;
        voltage     = +1.0;

		point_electrode_center_x = problem_x_start + (center_col - 1) * h_x;
		point_electrode_center_y = problem_y_start + (center_row - 150 - 1) * h_y;
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_12_electrolytic_tank_equal_pots_601_401 */

/*=========================================================================*/

int
lbmn_demo_xy_problem_13_2_inequal_plates_double_400(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 400;
    unsigned int problem_size_col = 400;
    double problem_x_start = 0.0;
    double problem_x_stop  = 399.0;
    double problem_y_start = 0.0;
    double problem_y_stop  = 399.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 400;
	    i_row_start   = 1;
	    i_row_stop    = 400;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*======================*
        * set electrode points  *
        *=======================*/
	
	    /* left plate */
	
	    i_col_start   = 181;
        i_col_stop    = 190;
	    i_row_start   = 101;
	    i_row_stop    = 300;
        boundary      = +1;
        voltage       = +1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("plate 1 allocated\n");

	    /* right plate */
	
        i_col_start = 211;
	    i_col_stop  = 220;
	    i_row_start = 141;
	    i_row_stop  = 250;
        boundary    = +2;
        voltage     = -1.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);

    printf("plate 2 allocated\n");

    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_13_2_inequal_plates_double_400 */

/*=========================================================================*/

int
lbmn_demo_xy_problem_14_quadrupole_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
	double delta_x = 0.0;/* parameters of centres of quadrupole  elements */
	double delta_y = 0.0;
	
    int ret_val = 0;

    /* instructions */

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	
		/*
		set quadrupole geometry
		*/
		delta_x = +50.0;
		delta_y = +50.0;
		
		
        /*
		set outer pipe
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* A - upper element */
	
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0 + delta_y; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
        /* B - bottom element */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0 - delta_y; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* C - left element */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = 0.0 - delta_x; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* D - right element */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = 0.0 + delta_x; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_14_quadrupole_401_401 */

/*===============================================================*/

int
lbmn_demo_xy_problem_15_cylindrical_capacitor_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
	
    int ret_val = 0;

    /* instructions */

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	
		
        /*
		set outer pipe
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /*
		set inner electrode
		*/
	
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

				
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_15_cylindrical_capacitor_401_401 */


/*=============================================================*/

int
lbmn_demo_xy_problem_16_quadrupole_v2_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
    int ret_val = 0;

	double R = 0.0;
	double alpha_zero = 0.0;
	int N = 0;
	int n = 0;
	
	double x_1 = 0.0;
	double x_2 = 0.0;
	double x_3 = 0.0;
	double x_4 = 0.0;
	
	double y_1 = 0.0;
	double y_2 = 0.0;
	double y_3 = 0.0;
	double y_4 = 0.0;
	
	
    /* instructions */

	/*
	* set (x,y) for centre of each electrode
	*/
	
	N = 4;/* quadrupole */
	alpha_zero = 0.25 * M_PI;/* angle = +45 degrees */
	R = 50.0; 
	
	n = 0;	
	x_1 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_1 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 1;	
	x_2 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_2 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 2;	
	x_3 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_3 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 3;	
	x_4 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_4 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	



		
        /*
		set outer casing
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* A - right element */
	
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_1; 
		point_electrode_center_y = y_1; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
        /* B - upper element */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_2; 
		point_electrode_center_y = y_2; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* C - left element */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_3; 
		point_electrode_center_y = y_3; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* D - bottom element */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_4; 
		point_electrode_center_y = y_4; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_16_quadrupole_v2_401_401 */

/*=============================================================*/

int
lbmn_demo_xy_problem_17_hexapole_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
    int ret_val = 0;

	double R = 0.0;
	double alpha_zero = 0.0;
	int N = 0;
	int n = 0;
	
	double x_1 = 0.0;
	double x_2 = 0.0;
	double x_3 = 0.0;
	double x_4 = 0.0;
	double x_5 = 0.0;
	double x_6 = 0.0;
	
	double y_1 = 0.0;
	double y_2 = 0.0;
	double y_3 = 0.0;
	double y_4 = 0.0;
	double y_5 = 0.0;
	double y_6 = 0.0;
	
	
    /* instructions */

	/*
	* set (x,y) for centre of each electrode
	*/
	
	N = 6;/* hexapole */
	alpha_zero = 0.0;
	R = 70.0; 
	
	n = 0;	
	x_1 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_1 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 1;	
	x_2 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_2 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 2;	
	x_3 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_3 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 3;	
	x_4 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_4 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 4;	
	x_5 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_5 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 5;	
	x_6 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_6 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	



		
        /*
		set outer casing
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* A  - element 1  */
	
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_1; 
		point_electrode_center_y = y_1; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
        /* B - element 2 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_2; 
		point_electrode_center_y = y_2; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* C - element 3 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_3; 
		point_electrode_center_y = y_3; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* D - element  4 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_4; 
		point_electrode_center_y = y_4; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


        /* E - element  5 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_5; 
		point_electrode_center_y = y_5; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


        /* F - element  6 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_6; 
		point_electrode_center_y = y_6; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_17_hexapole_401_401 */

/*===========================================================*/

int
lbmn_demo_xy_problem_18_skew_quadrupole_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
    int ret_val = 0;

	double R = 0.0;
	double alpha_zero = 0.0;
	int N = 0;
	int n = 0;
	
	/* double x_1 = 0.0; */
	double x_2 = 0.0;
	double x_3 = 0.0;
	/* double x_4 = 0.0; */
	double x_5 = 0.0;
	double x_6 = 0.0;
	
	/* double y_1 = 0.0; */
	double y_2 = 0.0;
	double y_3 = 0.0;
	/* double y_4 = 0.0; */
	double y_5 = 0.0;
	double y_6 = 0.0;
	
	
    /* instructions */

	/*
	* set (x,y) for centre of each electrode
	*/
	
	N = 6;/* hexapole */
	alpha_zero = 0.0;
	R = 70.0; 

	/*
	n = 0;	
	x_1 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_1 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	*/
	
	n = 1;	
	x_2 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_2 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 2;	
	x_3 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_3 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	/*
	n = 3;	
	x_4 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_4 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	*/
	
	n = 4;	
	x_5 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_5 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 5;	
	x_6 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_6 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);

	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	



		
        /*
		set outer casing
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* A  - element 1  - does not exist */

		
        /* B - element 2 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_2; 
		point_electrode_center_y = y_2; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* C - element 3 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_3; 
		point_electrode_center_y = y_3; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* D - element  4 - does not exist */



        /* E - element  5 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_5; 
		point_electrode_center_y = y_5; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


        /* F - element  6 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_6; 
		point_electrode_center_y = y_6; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_18_skew_quadrupole_401_401 */


/*=====================================================*/


int
lbmn_demo_xy_problem_19_octupole_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double distance = 0.0;
	double electrode_radius = 0.0;
	
	double h_x = 0.0;
	double h_y = 0.0;
	
	
	double point_electrode_center_x = 0.0;
	double point_electrode_center_y = 0.0;
	
	
	double point_x = 0.0;
	double point_y = 0.0;
	
    int ret_val = 0;

	double R = 0.0;
	double alpha_zero = 0.0;
	int N = 0;
	int n = 0;
	
	double x_1 = 0.0;
	double x_2 = 0.0;
	double x_3 = 0.0;
	double x_4 = 0.0;
	double x_5 = 0.0;
	double x_6 = 0.0;
	double x_7 = 0.0;
	double x_8 = 0.0;
	
	double y_1 = 0.0;
	double y_2 = 0.0;
	double y_3 = 0.0;
	double y_4 = 0.0;
	double y_5 = 0.0;
	double y_6 = 0.0;
	double y_7 = 0.0;
	double y_8 = 0.0;
	
	
    /* instructions */

	/*
	* set (x,y) for centre of each electrode
	*/
	
	N = 8;/* octupole  */
	alpha_zero = 0.0;
	R = 70.0; 

	
	n = 0;	
	x_1 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_1 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 1;	
	x_2 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_2 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 2;	
	x_3 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_3 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 3;	
	x_4 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_4 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 4;	
	x_5 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_5 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);
	
	n = 5;	
	x_6 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_6 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);

	n = 6;	
	x_7 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_7 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);

	n = 7;	
	x_8 = R * cos(alpha_zero + 2 * n * M_PI / (double)N);
	y_8 = R * sin(alpha_zero + 2 * n * M_PI / (double)N);


	h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
	h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
	
	
	/* try to allocate memory */
    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");

        /*============================*
        * set electrode points 		 *
        *=============================*/
	



		
        /*
		set outer casing
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
	
	
	
        /* A  - element 1 */
		
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_1; 
		point_electrode_center_y = y_1; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
		
		
        /* B - element 2 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_2; 
		point_electrode_center_y = y_2; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* C - element 3 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_3; 
		point_electrode_center_y = y_3; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

        /* D - element  4 */
		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_4; 
		point_electrode_center_y = y_4; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


        /* E - element  5 */

		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_5; 
		point_electrode_center_y = y_5; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


        /* F - element  6 */

		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_6; 
		point_electrode_center_y = y_6; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		
		/* G - element 7*/
		electrode_radius = 20.0;
		
        boundary    = +2;
        voltage     = +1.0;

		point_electrode_center_x = x_7; 
		point_electrode_center_y = y_7; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
		
		
		/* H - element 8*/
		
		electrode_radius = 20.0;
		
        boundary    = +3;
        voltage     = -1.0;

		point_electrode_center_x = x_8; 
		point_electrode_center_y = y_8; 
	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance <= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		
    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_19_octupole_401_401 */


/*=========================================================================*/

int
lbmn_demo_xy_problem_20_quadrupole_plates_401_401(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    /* daclaration of variables */

    unsigned int problem_size_row = 401;
    unsigned int problem_size_col = 401;
    double problem_x_start = -200.0;
    double problem_x_stop  = +200.0;
    double problem_y_start = -200.0;
    double problem_y_stop  = +200.0;

    double       voltage        = 0.0;
    unsigned int boundary       = 0;
    unsigned int i_row_start    = 0;
    unsigned int i_row_stop     = 0;
    unsigned int i_col_start    = 0;
    unsigned int i_col_stop     = 0;


	double h_x = 0.0;/* mesh step */
	double h_y = 0.0;

	unsigned int i_row_loop = 0;
	unsigned int i_col_loop = 0;
	
	double point_x = +0.0;
	double point_y = +0.0;
	
	
	double electrode_radius = 0.0;

	/* for setting round casing*/
	double point_electrode_center_x = +0.0;
	double point_electrode_center_y = +0.0;
	double distance = +0.0;
	
	double x_start = +0.0;
	double x_stop  = +0.0;
	double y_start = +0.0;
	double y_stop  = +0.0;
	
	
    int ret_val = 0;

    /* instructions */

    ret_val = lbmn_mesh_xy_problem_allocate(
        	    ptr_size_row,/*output*/
                ptr_size_col,
                ptr_x_start,
                ptr_x_stop,
                ptr_y_start,
                ptr_y_stop,
			    ptrptr_V,
			    ptrptr_Bnd,
			    ptrptr_E_x,
			    ptrptr_E_y,
			    ptrptr_E_1,
			    ptrptr_E_2,
                ptrptr_isolines_V,
                ptrptr_isolines_E_x,
                ptrptr_isolines_E_y,
                ptrptr_isolines_E_1,
                ptrptr_isolines_E_2,
			    ptrptr_workbench_colours_r,
			    ptrptr_workbench_colours_g,
			    ptrptr_workbench_colours_b,
			    ptrptr_workbench_bw_colours_r,
			    ptrptr_workbench_bw_colours_g,
			    ptrptr_workbench_bw_colours_b,
				
                problem_size_row,/* input */
                problem_size_col,
                problem_x_start,
                problem_x_stop,
                problem_y_start,
                problem_y_stop);

    if(ret_val !=0)
    {/* problem with memory allocation */

        printf("problem with problem creation --> ret_val == %i\n", ret_val);
        return -1;

    }/* problem with memory allocation */
    else
    {/* memory allocated */

        /*define problem */

        printf("memory allocated\n");
        

	
	    /* set the whole mesh to vacuum */
	
	    i_col_start   = 1;
        i_col_stop    = 401;
	    i_row_start   = 1;
	    i_row_stop    = 401;
        boundary      = 0;
        voltage       = +0.0;

	    lbmn_mesh_xy_D_set_rectangle_ij(
							*ptrptr_V,
							*ptrptr_Bnd,
							problem_size_row,/* size of mesh */
							problem_size_col,
							i_row_start,/* rectangle parameters */
							i_row_stop,
							i_col_start,
							i_col_stop,
							voltage, /*input rectangle values*/
							boundary);
    
        printf("mesh filled with vacuum\n");


		h_x = (problem_x_stop - problem_x_start)/(double)(problem_size_col -1);
		h_y = (problem_y_stop - problem_y_start)/(double)(problem_size_row -1);
        
		/*======================*
        * set electrode points  *
        *=======================*/

        /*
		set outer casing
		*/
        	
		electrode_radius = 190.0;
		
        boundary    = +1;
        voltage     = +0.0;

		point_electrode_center_x = 0.0; 
		point_electrode_center_y = 0.0; 	
	
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
				
				
				distance = sqrt((point_x - point_electrode_center_x)*(point_x - point_electrode_center_x) +
								(point_y - point_electrode_center_y)*(point_y - point_electrode_center_y));
				
				if(distance >= electrode_radius)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */
		

		/* set quadrupole segments */
		
		/* A - element 1 */
		
	    x_start = +100;
		x_stop  = +110.0;
		y_start = -30.0;
		y_stop  = +30.0;
		
        boundary      = +2;
        voltage       = +1.0;
	    
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
												
				
				if(	point_x >= x_start &&
					point_x <= x_stop  &&
					point_y >= y_start &&
					point_y <= y_stop
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


		/* B - element 2 */
		
	    x_start = -30.0;
		x_stop  = +30.0;
		y_start = +100.0;
		y_stop  = +110.0;
		
        boundary      = +3;
        voltage       = -1.0;
	    
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
												
				
				if(	point_x >= x_start &&
					point_x <= x_stop  &&
					point_y >= y_start &&
					point_y <= y_stop
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


		/* C - element 3 */
		
	    x_start = -110.0;
		x_stop  = -100.0;
		y_start = -30.0;
		y_stop  = +30.0;
		
        boundary      = +2;
        voltage       = +1.0;
	    
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
												
				
				if(	point_x >= x_start &&
					point_x <= x_stop  &&
					point_y >= y_start &&
					point_y <= y_stop
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */

		/* D - element 4 */
		
	    x_start = -30.0;
		x_stop  = +30.0;
		y_start = -110.0;
		y_stop  = -100.0;
		
        boundary      = +3;
        voltage       = -1.0;
	    
		for(i_row_loop = 1; i_row_loop <= problem_size_row; i_row_loop++)
		{
			for(i_col_loop = 1; i_col_loop <= problem_size_col; i_col_loop++)
			{
	
				point_x = problem_x_start + (i_col_loop - 1) * h_x;
				point_y = problem_y_start + (i_row_loop - 1) * h_y;
												
				
				if(	point_x >= x_start &&
					point_x <= x_stop 	&&
					point_y >= y_start &&
					point_y <= y_stop
				)
				{
					lbmn_mesh_xy_D_set_point_ij(
									*ptrptr_V,
									*ptrptr_Bnd,
									problem_size_row,/* size of mesh */
									problem_size_col,
									i_row_loop,/* point position (1,2, .. , n_row)*/
									i_col_loop,/* 1,2,..,n_col*/
									voltage, /*input point values*/
									boundary);
				}
				
			}/* loop col */
		}/* loop row */


    printf("plate 2 allocated\n");

    }/* memory allocated */
    return 0;
	
}/* lbmn_demo_xy_problem_20_quadrupole_plates_401_401 */

/* ==== end of file demo_xy_problems.c ==== */
