/* lbmn_anls_xy_D.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "lbmn_anls_xy_D.h"


int
lbmn_anls_xy_D_interp(  double V_1,
						double V_2,
						double V
)
{
	
	if(V_2 > V_1)
	{
		/*== E1 ==*/
	
		if(V < V_1)
		{/* alpha */
	
			return -1;
			
		}/* alpha */
		else if(V > V_2)
		{/* beta */
	
			return +1;
	
		}/* beta */
		else if( V <= 0.5*(V_1 + V_2) )
		{/* gamma */
			
			/* point belongs to gamma space */
			return 0;
			
		}/* gamma */
		else
		{/* delta */
	
			return +1;
			
		}/* delta */
		
	}/* V_2 > V_1 */
	else if (V_2 < V_1)
	{/*  V_2 < V_1  */
	
		/*== E2 ==*/

		if(V < V_2)
		{/* alpha */
	
			return -1;
			
		}/* alpha */
		else if(V > V_1)
		{/* beta */
	
			return +1;
	
		}/* beta */
		else if( V >= 0.5*(V_1 + V_2) )
		{/* gamma */
			
			/* point belongs to gamma space */
			return 0;
			
		}/* gamma */
		else
		{/* delta */
	
			return -1;
			
		}/* delta */
	
	}/*  V_2 < V_1  */
	else
	{/*  V_2 == V_1  */
	
		if(V == V_1)
		{
			/* point has value V_1 */
			return 0;
		}
		else
		{
			/* error ? */
			return -3;
		}
	
	}/*  V_2 == V_1  */
	
}/* lbmn_anls_xy_D_interp */

/*=========================================================================*/

int
lbmn_anls_xy_D_make_isoline(
			unsigned int*	ptr_mesh_isoline,/* output - mesh of isoline */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			val_isoline,/* value of isoline */
			unsigned int	index_isoline/* index of isoline */
			)
{
	/* declaration of variables */
	
	unsigned int icp = 0; /* index of central point */

	unsigned int i_row = 0;
	unsigned int i_col = 0;
	
	/* instructions */
	
	/* debug */
	/*
	printf("\n--- create_isoline: ---\n");
	printf("size_row      == %u\n", size_row);
	printf("size_col      == %u\n", size_col);
	printf("index_isoline == %u\n", index_isoline);
	*/
		
	/* data errors (wrong values) */
	if(ptr_mesh_isoline == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}
	
	
	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(index_isoline < 1)
	{
		return -6;
	}
	
	
	/*=== P1 ===*/
	icp = 0;

#ifdef DEBUG_ANLS_XY
		printf("P_1 - icp == %u\n", icp);
#endif

	if(*(ptr_Bnd + icp) == 0)
	{/* P1 is vacuum */

		/* try to detect isoline */
		if( (0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + 1),
						 val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col),
						 val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col + 1),
						 val_isoline))
		)
		{/* P1 belongs to isoline */
		
			*(ptr_mesh_isoline + icp) = index_isoline;
		
		}/* P1 belongs to isoline */

	}/* P1 is vacuum*/
	
	
	/*=== P2 ===*/
	
	for(i_col = 2; i_col <= size_col -1 ; i_col++)
	{/* loop P2 */

		icp = i_col - 1;

#ifdef DEBUG_ANLS_XY
		printf("P_2 - icp == %u\n", icp);
#endif
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P2 is vacuum */
	
			/* try to detect isoline */
			if( (0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col - 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col + 1),
							 val_isoline))
			)
			{/* P2 belongs to isoline */
			
				*(ptr_mesh_isoline + icp) = index_isoline;
			
			}/* P2 belongs to isoline */

		}/* P2 is vacuum*/

	}/* loop P2 */


	/*=== P3 ===*/
	
	icp = size_col - 1;
	
#ifdef DEBUG_ANLS_XY
		printf("P_3 - icp == %u\n", icp);
#endif

	if(*(ptr_Bnd + icp) == 0)
	{/* P3 is vacuum */

		/* try to detect isoline */
		if( (0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp - 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col - 1),
					 	val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + size_col),
						val_isoline))
		)
		{/* P3 belongs to isoline */
		
			*(ptr_mesh_isoline + icp) = index_isoline;
		
		}/* P3 belongs to isoline */

	}/* P3 is vacuum*/
	
	/*loop P4 - P5 - P6*/
	for(i_row = 2; i_row <= size_row - 1; i_row++)
	{/* loop P4 - P5 - P6 */
		
		/*=== P4 ===*/
		icp = (i_row - 1) * size_col;

#ifdef DEBUG_ANLS_XY
		printf("P_4 - icp == %u\n", icp);
#endif
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P4 is vacuum */

			/* try to detect isoline */
			if((0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col + 1),
							val_isoline))
			)
			{/* P4 belongs to isoline */
			
				*(ptr_mesh_isoline + icp) = index_isoline;
			
			}/* P4 belongs to isoline */

		}/* P4 is vacuum*/
		
		/*=== P5 ===*/

		for(i_col = 2; i_col <= size_col - 1; i_col++)
		{
			icp = (i_row - 1) * size_col + i_col - 1;
			
#ifdef DEBUG_ANLS_XY
		printf("P_5 - icp == %u\n", icp);
#endif
			if(*(ptr_Bnd + icp) == 0)
			{/* P5 is vacuum */

				/* try to detect isoline */
				if((0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp - size_col - 1),
							     val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
							    *(ptr_mesh + icp - size_col),
							      val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp - size_col + 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp - 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col - 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col + 1),
								val_isoline))
				)
				{/* P5 belongs to isoline */
				
					*(ptr_mesh_isoline + icp) = index_isoline;
				
				}/* P5 belongs to isoline */

			}/* P5 is vacuum*/
			
		}/* loop P5 */
		
		/*=== P6 ===*/


		icp = (i_row - 1) * size_col + size_col - 1;

#ifdef DEBUG_ANLS_XY
		printf("P_6 - icp == %u\n", icp);
#endif	

		if(*(ptr_Bnd + icp) == 0)
		{/* P6 is vacuum */

			/* try to detect isoline */
			if((0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							val_isoline))
			)
			{/* P6 belongs to isoline */
			
				*(ptr_mesh_isoline + icp) = index_isoline;
			
			}/* P6 belongs to isoline */

		}/* P6 is vacuum*/
		
	}/* loop P4 - P5 - P6 */
	
	/*=== P7 ===*/
	icp = size_col * (size_row - 1);

#ifdef DEBUG_ANLS_XY
		printf("P_7 - icp == %u\n", icp);
#endif
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P7 is vacuum */

		/* try to detect isoline */
		if((0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col + 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + 1),
						val_isoline))
		)
		{/* P7 belongs to isoline */
		
			*(ptr_mesh_isoline + icp) = index_isoline;
		
		}/* P7 belongs to isoline */

	}/* P7 is vacuum*/
	
	
	/*=== P8 ===*/
	
	for(i_col = 2; i_col <= size_col -1 ; i_col++)
	{/* loop P8 */

		icp = size_col * (size_row - 1) + i_col - 1;

#ifdef DEBUG_ANLS_XY
		printf("P_8 - icp == %u\n", icp);
#endif

		if(*(ptr_Bnd + icp) == 0)
		{/* P8 is vacuum */

			/* try to detect isoline */
			if( 
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							val_isoline))
			)
			{/* P8 belongs to isoline */
			
				*(ptr_mesh_isoline + icp) = index_isoline;
			
			}/* P8 belongs to isoline */

		}/* P8 is vacuum*/

	}/* loop P8 */


	/*=== P9 ===*/
	
	icp = size_row * size_col - 1;

#ifdef DEBUG_ANLS_XY
		printf("P_9 - icp == %u\n", icp);
#endif
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P9 is vacuum */

		/* try to detect isoline */
		if( 
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col - 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp - 1),
						val_isoline))
			
		)
		{/* P9 belongs to isoline */
		
			*(ptr_mesh_isoline + icp) = index_isoline;
		
		}/* P9 belongs to isoline */

	}/* P9 is vacuum*/
	
	/* lrt's assume succcess (input data is correct). No line collisions were detected */
	return 0;
	
}/* lbmn_lbmn_anls_xy_D_make_isoline */


/*=====================================================================*/

int
lbmn_anls_xy_D_make_isoline_strict(
			unsigned int*   ptr_err_row,
			unsigned int*   ptr_err_col,
			unsigned int*	ptr_mesh_isoline,/* output - mesh of isolines */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			val_isoline,/* value of isoline */
			unsigned int	index_isoline/* index of isoline */
			)
{
	/* declaration of variables */
	
	unsigned int icp = 0; /* index of central point */

	unsigned int i_row = 0;
	unsigned int i_col = 0;
	
	/* instructions */
	
	/* data errors (wrong values) */
	if(ptr_mesh_isoline == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}
	
	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(index_isoline < 1)
	{
		return -6;
	}
	
	
	
	/* if no input data erroros, start to traverse mesh */
	
	/* let's assume there is no problematic nodes */
	*ptr_err_row = 0;
	*ptr_err_col = 0;
	
	/*=== P1 ===*/
	icp = 0;
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P1 is vacuum */

		/* try to detect isoline */
		if( (0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + 1),
						 val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col),
						 val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col + 1),
						 val_isoline))
		)
		{/* P1 belongs to isoline */
		
			if(*(ptr_mesh_isoline + icp) == 0)
			{/* this point is not busy */
		
				/* we can set another isoline point*/
				*(ptr_mesh_isoline + icp) = index_isoline;
				
			}/* this point is not busy */
			else
			{/* this point is busy */
		
				/* error - point is busy */
				
				/*set indices of first problematic node */
				/* P1 (1,1) */
				*ptr_err_row = 1;
				*ptr_err_col = 1;
				
				return -7;
				
			}/* this point is busy */
		
		}/* P1 belongs to isoline */

	}/* P1 is vacuum*/
	
	
	/*=== P2 ===*/
	
	for(i_col = 2; i_col <= size_col -1 ; i_col++)
	{/* loop P2 */

		icp = i_col - 1;
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P2 is vacuum */

			/* try to detect isoline */
			if( (0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col - 1),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							 val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col + 1),
							 val_isoline))
			)
			{/* P2 belongs to isoline */
			
				if(*(ptr_mesh_isoline + icp) == 0)
				{/* this point is not busy */
			
					/* we can set another isoline point*/	
					*(ptr_mesh_isoline + icp) = index_isoline;
					
				}/* this point is not busy */
				else
				{/* this point is busy */
		
					/* error - point is busy */
					/*set indices of first problematic node */
				
					/* P2 (1, i_col) */
					*ptr_err_row = 1;
					*ptr_err_col = i_col;
				
					return -7;
				
				}/* this point is busy */
			
			}/* P2 belongs to isoline */

		}/* P2 is vacuum*/

	}/* loop P2 */


	/*=== P3 ===*/
	
	icp = size_col - 1;
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P3 is vacuum */

		/* try to detect isoline */
		if( (0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp - 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp + size_col - 1),
					 	val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + size_col),
						val_isoline))
		)
		{/* P3 belongs to isoline */
		
			if(*(ptr_mesh_isoline + icp) == 0)
			{
				/* we can set another isoline point */
				*(ptr_mesh_isoline + icp) = index_isoline;
		
			}/* this point is not busy */
			else
			{/* this point is busy */
		
				/* error - point is busy */
				/*set indices of first problematic node */
				
				/* P3 (1, size_col)*/
				*ptr_err_row = 1;
				*ptr_err_col = size_col;
				
				return -7;
				
			}/* this point is busy */
				
		}/* P3 belongs to isoline */

	}/* P3 is vacuum*/
	
	/*loop P4 - P5 - P6*/
	for(i_row = 2; i_row <= size_row - 1; i_row++)
	{/* loop P4 - P5 - P6 */
		
		/*=== P4 ===*/
		icp = (i_row - 1) * size_col;
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P4 is vacuum */

			/* try to detect isoline */
			if((0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col + 1),
							val_isoline))
			)
			{/* P4 belongs to isoline */
			
				if(*(ptr_mesh_isoline + icp) == 0)
				{/* this point is not busy */ 
					
					/* we can set another isoline point */
					*(ptr_mesh_isoline + icp) = index_isoline;
					
				}/* this point is not busy */
				else
				{/* this point is busy */
		
					/* error - point is busy */
					/*set indices of first problematic node */
				
					/* P4 (i_row, 1)*/
					*ptr_err_row = i_row;
					*ptr_err_col = 1;
				
					return -7;
				
				}/* this point is busy */
				
			}/* P4 belongs to isoline */

		}/* P4 is vacuum*/
		
		/*=== P5 ===*/

		for(i_col = 2; i_col <= size_col - 1; i_col++)
		{
			icp = (i_row - 1) * size_col + i_col - 1;
			
			if(*(ptr_Bnd + icp) == 0)
			{/* P5 is vacuum */

				/* try to detect isoline */
				if( (0== lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp - size_col - 1),
							     val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
							    *(ptr_mesh + icp - size_col),
							      val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col + 1),
							val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp - 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col - 1),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col),
								val_isoline))
					||
					(0 == lbmn_anls_xy_D_interp(
								*(ptr_mesh + icp),
								*(ptr_mesh + icp + size_col + 1),
								val_isoline))
				)
				{/* P5 belongs to isoline */
				
					if(*(ptr_mesh_isoline + icp) == 0)
					{/* this point is not busy */ 
						
						/* we can set another isoline point */
						*(ptr_mesh_isoline + icp) = index_isoline;
						
					}/* this point is not busy */
					else
					{/* this point is busy */
			
						/* error - point is busy */
						/*set indices of first problematic node */
					
						/* P5 (i_row, i_col)*/
						*ptr_err_row = i_row;
						*ptr_err_col = i_col;
					
						return -7;
					
					}/* this point is busy */

				}/* P5 belongs to isoline */

			}/* P5 is vacuum*/
			
		}/* loop P5 */
		
		/*=== P6 ===*/

		icp = (i_col - 1) * size_col + size_col - 1;
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P6 is vacuum */

			/* try to detect isoline */
			if( (0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + size_col),
							val_isoline))
			)
			{/* P6 belongs to isoline */
			
				*(ptr_mesh_isoline + icp) = index_isoline;
			
				if(*(ptr_mesh_isoline + icp) == 0)
				{/* this point is not busy */ 
						
					/* we can set another isoline point */
					*(ptr_mesh_isoline + icp) = index_isoline;
						
				}/* this point is not busy */
				else
				{/* this point is busy */
			
					/* error - point is busy */
					/*set indices of first problematic node */
				
					/* P6 (i_row, size_col)*/
					*ptr_err_row = i_row;
					*ptr_err_col = size_col;
					
					return -7;
					
				}/* this point is busy */
			
			}/* P6 belongs to isoline */

		}/* P6 is vacuum*/
		
	}/* loop P4 - P5 - P6 */
	
	/*=== P7 ===*/
	icp = size_col * (size_row - 1);
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P7 is vacuum */

		/* try to detect isoline */
		if(	(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col + 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp + 1),
						val_isoline))
		)
		{/* P7 belongs to isoline */
		
			if(*(ptr_mesh_isoline + icp) == 0)
			{/* this point is not busy */ 
						
				/* we can set another isoline point */
				*(ptr_mesh_isoline + icp) = index_isoline;
						
			}/* this point is not busy */
			else
			{/* this point is busy */
			
				/* error - point is busy */
				/*set indices of first problematic node */
					
				/* P7 (size_row, 1)*/
				*ptr_err_row = size_row;
				*ptr_err_col = 1;
			
				return -7;
				
			}/* this point is busy */
		
		}/* P7 belongs to isoline */

	}/* P7 is vacuum*/
	
	
	/*=== P8 ===*/
	
	for(i_col = 2; i_col <= size_col -1 ; i_col++)
	{/* loop P8 */

		icp = size_col * (size_row - 1) + i_col - 1;
		
		if(*(ptr_Bnd + icp) == 0)
		{/* P8 is vacuum */

			/* try to detect isoline */
			if( (0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - size_col + 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp - 1),
							val_isoline))
				||
				(0 == lbmn_anls_xy_D_interp(
							*(ptr_mesh + icp),
							*(ptr_mesh + icp + 1),
							val_isoline))
			)
			{/* P8 belongs to isoline */
			
				
				if(*(ptr_mesh_isoline + icp) == 0)
				{/* this point is not busy */ 
						
					/* we can set another isoline point */
					*(ptr_mesh_isoline + icp) = index_isoline;
						
				}/* this point is not busy */
				else
				{/* this point is busy */
			
					/* error - point is busy */
					/*set indices of first problematic node */
					
					/* P8 (size_row, i_col)*/
					*ptr_err_row = size_row;
					*ptr_err_col = i_col;
					
					return -7;
					
				}/* this point is busy */
			
			}/* P8 belongs to isoline */

		}/* P8 is vacuum*/

	}/* loop P8 */


	/*=== P9 ===*/
	
	icp = size_row * size_col - 1;
	
	if(*(ptr_Bnd + icp) == 0)
	{/* P9 is vacuum */

		/* try to detect isoline */
		if( (0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col - 1),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
				        *(ptr_mesh + icp - size_col),
						val_isoline))
			||
			(0 == lbmn_anls_xy_D_interp(
						*(ptr_mesh + icp),
						*(ptr_mesh + icp - 1),
						val_isoline))
			
		)
		{/* P9 belongs to isoline */
		
			*(ptr_mesh_isoline + icp) = index_isoline;
		
							if(*(ptr_mesh_isoline + icp) == 0)
					{/* this point is not busy */ 
						
						/* we can set another isoline point */
						*(ptr_mesh_isoline + icp) = index_isoline;
						
					}/* this point is not busy */
					else
					{/* this point is busy */
			
						/* error - point is busy */
						/*set indices of first problematic node */
					
						/* P9 (size_row, size_col)*/
						*ptr_err_row = size_row;
						*ptr_err_col = size_col;
					
						return -7;
					
					}/* this point is busy */
					
		}/* P9 belongs to isoline */

	}/* P9 is vacuum*/
	
	/* if no errors - return success */
	return 0;
	
}/* lbmn_anls_xy_D_make_isoline_strict */


/*===========================================================*/


int
lbmn_anls_xy_D_create_all_isolines(
				unsigned int* 	ptr_mesh_isoline,/* output */
				double*			ptr_mesh,/* input */
				unsigned int*	ptr_Bnd,				
				unsigned int 	size_row,
				unsigned int 	size_col,
				double 			V_min,
				double			V_max,
				unsigned int	number_of_isolines
				)
{
	
	/* declaration of variables  */
	
	double val 		= 0.0;
	double val_step = 0.0;
	
	unsigned int index = 0;
	
	/* instructions */
	
	if(ptr_mesh_isoline == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}
	
	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(V_max <= V_min)
	{
		return -6;
	}
	
	if(number_of_isolines < 2)
	{
		return -7;
	}
	
	
		
	/* if no input data errors, let's go */
	
	/* set the isoline data table to zero */
	for(index = 1; index <= size_row * size_col; index++)
	{
		*(ptr_mesh_isoline + index - 1) = 0;
		
	}
	
	/* determine isoline step (uniform distribution) */
	val_step = (V_max - V_min ) / (double)(number_of_isolines + 1);
	
	
	/* set all the isolines */
	for(index = 1; index <= number_of_isolines; index++)
	{
		/* determine value of another isoline */
		val = V_min + (double)index * val_step;
		
        printf("creating isoline for: V == %+f...", val);

		if(0 != lbmn_anls_xy_D_make_isoline(
						ptr_mesh_isoline,/* output - mesh of isoline */
						ptr_mesh,/* input */
						ptr_Bnd,
						size_row,
						size_col,
						val,/* value of isoline */
						index/* index of isoline */
			)
		)
		{/* failed to create isoline */
		
			printf("failed\n");
			
			/* abend - problem with call make_isoline */
			return -8;
		}/* failed to create isoline */
		else
        {/* isoline created */
	
            printf("ok\n");
			
        }/* isoline created */
		
		
	}/* loop for all isolines */

	/* if no abend behaviour - return success */
	return 0;
	
}/* lbmn_anls_xy_D_create_all_isolines */


/*===========================================================*/



int
lbmn_anls_xy_D_in_range(double V_1,
						double V_2,
						double V
)
{
	
	if(V_2 < V_1)
	{
		return -2;
	}
	
	if (V >= V_1 && V <= V_2)
	{
		/* V is in range V_1 ... V_2 */
		return 0;
	}
	else
	{
		/* V is out of range */
		return -1;
	}
	
		
}/* lbmn_anls_xy_D_in range */

/*=========================================================================*/

int
lbmn_anls_xy_D_make_range_map(
			unsigned int*	ptr_mesh_range_map,/* output - mesh of isoline */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double			V_1, /* value of range: V_1 ... V_2  */
			double			V_2,
			unsigned int	index_range/* index of isoline */
			)
{
	/* declaration of variables */
	
	unsigned int icp = 0; /* index of central point */
	
	/* instructions */
	
	/* data errors (wrong values) */
	
	if(ptr_mesh_range_map == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}
	
	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(V_1 >= V_2)
	{
		/* wrong range definition */
		return -6;
	}
	
	if(index_range < 1)
	{
		return -7;
	}
	
	
	/* if no input errors, traverse the mesh */
	
	for(icp = 0; icp <= size_row * size_col - 1; icp++)
	{
		
		if(*(ptr_Bnd + icp) == 0)
		{/* node is vacuum */

			/* try to detect range */
			if( 0 == lbmn_anls_xy_D_in_range(
									V_1,
									V_2,
									*(ptr_mesh + icp)
									)
			)
			{/* point belongs to range */
		
				/* mark node to appropriate index */
				*(ptr_mesh_range_map + icp) = index_range;
		
			}/* point belongs to range */

		}/* node is vacuum*/
	}
	
	/* if no errors, let'a assume success (no line collisions were detected) */
	return 0;
	
}/* lbmn_lbmn_anls_xy_D_make_range_map */


/*=====================================================================*/

int
lbmn_anls_xy_D_make_range_map_strict(
			unsigned int*   ptr_err_row,
			unsigned int*   ptr_err_col,
			unsigned int*	ptr_mesh_range_map,/* output - mesh of isolines */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			V_1,/* value of range */
			double			V_2,
			unsigned int	index_range/* index of isoline */
			)
{
	/* declaration of variables */
	unsigned int i_row = 0;
	unsigned int i_col = 0;
	
	unsigned int icp = 0; /* index of central point */
	
	/* instructions */
	
	/* input data errors (wrong values) */
	if(ptr_mesh_range_map == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}
	
	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(V_1 >= V_2)
	{
		/* wrong range definition */
		return -6;
	}
	
	if(index_range < 1)
	{
		return -7;
	}
	
	
	/* let's assume there is no problematic nodes */
	*ptr_err_row = 0;
	*ptr_err_col = 0;
	
	
	/* traverse the entire mesh row by row */
	
	for(i_row = 1; i_row <= size_row ; i_row++)
	{	
		for(i_col = 1; i_col <= size_col; i_col++)
		{			
	
			icp = (i_row - 1) * size_col + i_col - 1;
			
			if(*(ptr_Bnd + icp) == 0)
			{/* point is vacuum */

				/* try to detect range */
				if( 0 == lbmn_anls_xy_D_in_range(
									V_1,
									V_2,
									*(ptr_mesh + icp)
									) 
				)
				{/* node belongs to range */
				
					if(*(ptr_mesh_range_map + icp) == 0)
					{/* this point is not busy */
				
						/* we can set another isoline point*/
						*(ptr_mesh_range_map + icp) = index_range;
						
					}/* this point is not busy */
					else
					{/* this point is busy */
				
						/* error - point is busy */
						
						/*set output indices of first problematic node */
						*ptr_err_row = i_row;
						*ptr_err_col = i_col;
						
						/* return error */
						return -8;
						
					}/* this point is busy */
				
				}/* point belongs to range */
				
			}/* node is vacuum*/
			
		}/* loop i_col */
		
	}/* loop i_row */
	
	/* if no errors, return success */
	return 0;
	
	
}/* lbmn_anls_xy_D_make_range_map_strict */

/*==================================================================*/


int
lbmn_anls_xy_D_fprintf_mesh_isolines_bw_netpbm_P3(
            const char*    	filename,
            unsigned int*  	ptr_Bnd,
			unsigned int*	ptr_mesh_isoline,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned char 	colour_background_red,/* colour - background */
			unsigned char 	colour_background_green,
			unsigned char 	colour_background_blue,
			unsigned char* 	ptr_isoline_colours_red,/* colours of isolines */
			unsigned char* 	ptr_isoline_colours_green,
			unsigned char* 	ptr_isoline_colours_blue,
			unsigned int 	number_of_isoline_colours,
			unsigned char* 	ptr_electrode_colours_red,/* colours of electrodes */
			unsigned char* 	ptr_electrode_colours_green,
			unsigned char* 	ptr_electrode_colours_blue)

{
	/* definition of variables */
	
	FILE* ptr_file;
	
	unsigned int i_row = 0;
	unsigned int i_col = 0;
	
	unsigned int icp = 0;
	unsigned int index_isoline = 0;
	unsigned int boundary = 0;
	
	/* instructions */
	
	
	
	/* check input data correctness */
	
	if(filename == NULL)
	{
		return -1;
	}
	
	if(ptr_Bnd == NULL)
	{
		return -2;
	}
	
	if(ptr_mesh_isoline == NULL)
	{
		return -3;
	}
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(	ptr_isoline_colours_red == NULL ||
		ptr_isoline_colours_green == NULL ||
		ptr_isoline_colours_blue == NULL 
	)
	{
		return -6;
	}
	
	if(number_of_isoline_colours < 1)
	{
		return -7;
	}
	
	if(	ptr_electrode_colours_red == NULL ||
		ptr_electrode_colours_green == NULL ||
		ptr_electrode_colours_blue == NULL
	)
	{
		return -8;
	}
	

	/* if no input data errors, let's go on */


	/* try to open output file */
	ptr_file = fopen (filename,"w");

    if (ptr_file == NULL)
    {
        /* could not open output file */
        return -9;
    }
    else
    {/* output file opened successfully */

        /* write file header */

        fprintf (ptr_file, "P3\n %u %u\n 255\n", size_col, size_row);
 
        if(ferror(ptr_file))
        {
            /* could not write header */
            fclose(ptr_file);
            return -10;
        }
        else
        {/* file header written successfully */
	
			
			
			for(i_row = size_row; i_row >= 1; i_row--)
            {
                for(i_col = 1; i_col <= size_col; i_col++)
                {

					/* determine index of central point */
					icp = (i_row - 1) * size_col + i_col -1;
					
                    /* debug */
                    /* printf("icp == %u\n", icp); */

					/* determine boundary value*/
					boundary = *(ptr_Bnd + icp);
									
									
					if(boundary > 0)
					{/* mesh point is electrode */
				
						fprintf(ptr_file, "%3u %3u %3u\n",
									*(ptr_electrode_colours_red   + boundary),
									*(ptr_electrode_colours_green + boundary),
									*(ptr_electrode_colours_blue  + boundary));
                        
						
						if(ferror(ptr_file))
						{
							/* could not write data to file (electrode) */
							fclose(ptr_file);
							return -11;
						}
				
					}/* mesh point is electrode */
					else
					{/* mesh point is vacuum */
				
						index_isoline = *(ptr_mesh_isoline + icp);
						
						if(index_isoline == 0 )
						{/* vacuum without isoline */
					
					
							/* try to write background colour */
							fprintf(ptr_file, "%3u %3u %3u\n",	colour_background_red,
																colour_background_green,
																colour_background_blue);

                            /*
                            printf("point is background -> %3u, %3u, %3u\n", colour_background_red,
   												    			            colour_background_green,
																            colour_background_blue);
                            */

							if(ferror(ptr_file))
							{
								fclose(ptr_file);
								return -12;
							}
							
						}/* vacuum without isoline*/
						else
						{/* vacuum with isoline */
					

							/* try to write isoline data to file */
							fprintf(ptr_file, "%3u %3u %3u\n",	*(ptr_isoline_colours_red   + index_isoline),
																*(ptr_isoline_colours_green + index_isoline),
																*(ptr_isoline_colours_blue  + index_isoline)
																);
														

#ifdef DEBUG_ANLS_XY
                            printf("point is isoline %3u -> %3u, %3u, %3u\n",   index_isoline,
                                                                                *(ptr_isoline_colours_red   + index_isoline),
												    			                *(ptr_isoline_colours_green + index_isoline),
																                *(ptr_isoline_colours_blue  + index_isoline));
#endif

							if(ferror(ptr_file))
							{
								fclose(ptr_file);
								return -13;
							}					
						}/* vacuum with isoline */
												
					}/* mesh point is vacuum */

                }/* loop i_col*/

            }/* loop i_row*/

            /* success, now we can close file */
            fclose(ptr_file);	
			
			
		}/* file header written successfully */
		
	}/*output file opened successfully*/
	
	
	/* if no errors - return success */
	return 0;
	
}/* lbmn_anls_xy_D_fprintf_mesh_isolines_bw_netpbm_P3 */

/* =============================================================== */


/* TODO TODO TODO TODO !!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*
int
lbmn_anls_xy_D_fprintf_mesh_isolines_colormap_netpbm_P3(
)
{
	return -1;
}

*/		

/*=================================================================*/

int
lbmn_anls_xy_D_goto_ij(
				double*			ptr_pos_x,/*output*/
				double*			ptr_pos_y,
				unsigned int*	ptr_Bnd,
				double*			ptr_V,
				double*			ptr_E_x,
				double*			ptr_E_y,
				double*			ptr_mesh_V,/* input */
				unsigned int*	ptr_mesh_Bnd,
				unsigned int	size_row,
				unsigned int	size_col,
				double			x_min,
				double			x_max,
				double			y_min,
				double			y_max,
				unsigned int	i_row,
				unsigned int	i_col				
				)
{
	/* declaration of avariables */

	unsigned int 	icp	= 0;/* index if out point on mesh ptr_mesh_V and ptr_mesh_Bnd */

	double	h_x = 0.0;/* mesh step */
	double	h_y = 0.0;

	double	tmp_pos_x		= 0.0;
	double	tmp_pos_y		= 0.0;
	unsigned int tmp_Bnd 	= 0;
	double tmp_V 			= 0.0;
	double tmp_E_x 			= 0.0;
	double tmp_E_y 			= 0.0;
	
	/* instructions */
	
	/* check if function parameters are correct */

	if(ptr_pos_x == NULL)
	{
		return -1;
	}
	
	if(ptr_pos_y == NULL)
	{
		return -2;
	}
	
	if(ptr_mesh_V == NULL)
	{
		return -3;
	}
	
	if(ptr_mesh_Bnd == NULL)
	{
		return -4;
	}
	
	if(size_row < 3)
	{
		return -5;
	}
	
	if(size_col < 3)
	{
		return -6;
	}
	
	
	if(x_max <= x_min)
	{
		return -7;
	}
	
	if(y_max <= y_min)
	{
		return -8;
	}
		
	if(i_row < 1)
	{
		return -9;
	}
	
	if(i_row > size_row)
	{
		return -10;
	}
	
	if(i_col < 1)
	{
		return -11;
	}
	
	if(i_col > size_col)
	{
		return -12;
	}

	
	/* else - input data seems to be ok */

	/* set outputs to zero */
	*ptr_pos_x	= 0.0;
	*ptr_pos_y	= 0.0;
	*ptr_Bnd 	= 0;
	*ptr_V 		= 0.0;
	*ptr_E_x 	= 0.0;
	*ptr_E_y 	= 0.0;
	
	
	/* determine mesh step */
	h_x = (x_max - x_min) / (size_col - 1);
	h_y = (y_max - y_min) / (size_row - 1);



	/* determine pos_x and pos_y */
	tmp_pos_x = x_min + (i_col - 1) * h_x;
	tmp_pos_y = y_min + (i_row - 1) * h_y;
	
	/* determine Bnd and V */
	
	icp = (i_row - 1) * size_col + i_col - 1;
	
	tmp_V 	= *(ptr_mesh_V   + icp);
	tmp_Bnd = *(ptr_mesh_Bnd + icp);
	
	
	
	/* set E_x and E_y	*/
	
	if(i_row == 1)
	{
		
		if(i_col == 1)
		{
			/* point  (1, 1) */
			
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp)) / h_y;
			
		}/* if i_col == 1 */
		else if(i_col < size_col)
		{/* i_col == 2, .., (size_col - 1) */
	
			/* point (1, i_col) */
		
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp)) / h_y;
		
		}/* i_col == 2, .., (size_col - 1) */
		else
		{/* i_col == size_col */
	
			/* point (1, size_col) */
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp)) / h_y;
		
		}/* i_col == size_col */
		
	}/* if i_row == 1 */
	else if(i_row < size_row)
	{/* i_row == 2, .., (size_row - 1) */
		
		if(i_col == 1)
		{
			/* point  (i_row, 1) */
			
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp )) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp - size_col)) / h_y;
			
		}/* if i_col == 1 */
		else if(i_col < size_col)
		{/* i_col == 2, .., (size_col - 1) */
	
			/* point (1, i_col) */
		
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp - size_col)) / h_y;
		
		}/* i_col == 2, .., (size_col - 1) */
		else
		{/* i_col == size_col */
	
			/* point (1, size_col) */
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp + size_col) - *(ptr_mesh_V + icp - size_col)) / h_y;
		
		}/* i_col == size_col */
		
	}/* i_row == 2, .., (size_row - 1) */
	else
	{/* i_row == size_row */

		if(i_col == 1)
		{
			/* point  (size_row, 1) */
			
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp )) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - size_col)) / h_y;
			
		}/* if i_col == 1 */
		else if(i_col < size_col)
		{/* i_col == 2, .., (size_col - 1) */
	
			/* point (size_row, i_col) */
		
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp + 1) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - size_col)) / h_y;
		
		}/* i_col == 2, .., (size_col - 1) */
		else
		{/* i_col == size_col */
	
			/* point (size_row, size_col) */
			tmp_E_x = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - 1)) / h_x;
			
			tmp_E_y = 0.5 * (*(ptr_mesh_V + icp) - *(ptr_mesh_V + icp - size_col)) / h_y;
		
		}/* i_col == size_col */
		
	}/* i_row == size_row */

	/* now E_x and E_y are determined */
	
	
	/* set output data  and end function */
	*ptr_pos_x	= tmp_pos_x;
	*ptr_pos_y	= tmp_pos_y;
	*ptr_Bnd 	= tmp_Bnd;
	*ptr_V 		= tmp_V;
	*ptr_E_x 	= tmp_E_x;
	*ptr_E_y 	= tmp_E_y;

	return 0;	
}/* lbmn_anls_xy_D_goto_ij */

/*==================================================================*/

int
lbmn_anls_xy_D_goto_xy(
				unsigned int*	ptr_pos_x,
				unsigned int*	ptr_pos_y,
				unsigned int*	ptr_Bnd,
				double*			ptr_V,
				double*			ptr_E_x,
				double*			ptr_E_y,
				double*			ptr_mesh_V,/* input */
				unsigned int*	ptr_mesh_Bnd,
				unsigned int	size_row,
				unsigned int	size_col,
				double			x_min,
				double			x_max,
				double			y_min,
				double			y_max,
				double			pos_x,
				double			pos_y
				)
{
	/* declaration of avariables */

	/*unsigned int 	icp	= 0; index if out point on mesh ptr_mesh_V and ptr_mesh_Bnd */

	double	h_x = 0.0;/* mesh step */
	double	h_y = 0.0;

	unsigned int tmp_pos_x	= 0;
	unsigned int tmp_pos_y	= 0;
	unsigned int tmp_Bnd 	= 0;
	double tmp_V 			= 0.0;
	double tmp_E_x 			= 0.0;
	double tmp_E_y 			= 0.0;
	
	/* instructions */
	
	
	/* check if function parameters are correct */
	if(ptr_pos_x == NULL)
	{
		return -1;
	}
	
	if(ptr_pos_y == NULL)
	{
		return -2;
	}
	
	if(ptr_mesh_V == NULL)
	{
		return -3;
	}
	
	if(ptr_mesh_Bnd == NULL)
	{
		return -4;
	}
	
	if(size_row < 3)
	{
		return -5;
	}
	
	if(size_col < 3)
	{
		return -6;
	}
	
	
	if(x_max <= x_min)
	{
		return -7;
	}
	
	if(y_max <= y_min)
	{
		return -8;
	}
		

	
	/* else - input data seems to be ok */

	/* set outputs to zero */
	*ptr_Bnd 	= 0;
	*ptr_V 		= 0.0;
	*ptr_E_x 	= 0.0;
	*ptr_E_y 	= 0.0;
	
	/*
	* check position on mesh and set another outputs
	* 1 - below min
	* 2 - between min ... max
	* 3 - abowe max
	*/
	
	/* check in x dimestion */
	if(pos_x < x_min)
	{
		/* below x_min */
		tmp_pos_x = 1;
	}
	else if(pos_x <= x_max)
	{
		/* between x_min, ... , x_max */
		tmp_pos_x = 2;
	}
	else
	{
		/* abowe x_max */
		tmp_pos_x = 3;
	}
	
	/* checj in y dimension */
	if(pos_y < y_min)
	{
		/* below y_min */
		tmp_pos_y = 1;
	}
	else if(pos_y <= y_max)
	{
		/* between y_min, ... , y_max */
		tmp_pos_y = 2;
	}
	else
	{
		/* abowe y_max */
		tmp_pos_y = 3;
	}

	/* if point is inside mesh, we can do calculations */
	if(tmp_pos_x == 2 && tmp_pos_y == 2)
	{
		/* determine mesh step */
		h_x = (x_max - x_min) / (size_col - 1);
		h_y = (y_max - y_min) / (size_row - 1);

		printf("h_x == %f\nh_y == %f\n", h_x, h_y);
		/*
		******** TODO ALL THE JOB ********************
		*/
		
	}/* point is inside mesh (x_min <= x <= x_max) && (y_min <= y <= y_max) */
	/*
	* else
	* point is outside a mesh - nothing to do
	*/


	/* now E_x and E_y are determined */
	
	
	/* set output data  and end function */
	*ptr_pos_x	= tmp_pos_x;
	*ptr_pos_y	= tmp_pos_y;
	*ptr_Bnd 	= tmp_Bnd;
	*ptr_V 		= tmp_V;
	*ptr_E_x 	= tmp_E_x;
	*ptr_E_y 	= tmp_E_y;

	return 0;	
}/* lbmn_anls_xy_D_goto_xy */


/* ==== end of file lbmn_anls_xy_D.c ==== */
