/* lbmn_anls_xy_D.h
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef LBMN_ANLS_XY_D_H_
#define LBMN_ANLS_XY_D_H_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/*=== unlock define to enable debug messages ===*/

/*
#define DEBUG_ANLS_XY
*/

int
lbmn_anls_xy_D_interp(  double V_1,
						double V_2,
						double V);



/* 
* no collision checking 
* return value:
*
*  (-6).. (-1) if wrong input data values (NULL pointers, size < 3, line index < 1 )
*    0 else 
*/
int
lbmn_anls_xy_D_make_isoline(
			unsigned int*	ptr_mesh_isoline,/* output - mesh of isoline */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			val_isoline,/* value of isoline */
			unsigned int	index_isoline/* index of isoline */
			);
	
/*
* isoline mesh must be set to 0 (all indices)
* if 2 points belongs to the same interpolated mesh - error is returned (-1) and indices of problematic nodes.s
*
* return value:
*
*  (-6) .. (-1) if wrong input data values
*    0 if success
*  (-7) if collisions with other isoline has been detected
*/
int
lbmn_anls_xy_D_make_isoline_strict(
			unsigned int*   ptr_err_row,/* output - index if first problematic node (row/col) if error occured */
			unsigned int*   ptr_err_col,
			unsigned int*	ptr_mesh_isoline,/* output - mesh of isoline */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			val_isoline,/* value of isoline */
			unsigned int	index_isoline/* index of isoline */
			);
			


int
lbmn_anls_xy_D_create_all_isolines(
				unsigned int* 	ptr_mesh_isoline,/* output */
				double*			ptr_mesh,/* input */
				unsigned int*	ptr_Bnd,				
				unsigned int 	sie_row,
				unsigned int 	size_col,
				double 			V_min,
				double			V_max,
				unsigned int	number_of_isolines
				);

/*
if(V_2 < V_1)
	return -2; error

if (V1 <= V <= V2)
	return 0   // V in range V_1 ... V_2
else
	return -1  // V not in range
*/
int
lbmn_anls_xy_D_in_range(double V_2,
						double V_1,
						double V);
						
						



int
lbmn_anls_xy_D_make_range_map(
			unsigned int*	ptr_mesh_range_map,/* output - mesh of isoline */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double			V_1, /* value of range: V_1 ... V_2  */
			double			V_2,
			unsigned int	index_range/* index of isoline */
			);

int
lbmn_anls_xy_D_make_range_map_strict(
			unsigned int*   ptr_err_row,
			unsigned int*   ptr_err_col,
			unsigned int*	ptr_mesh_range_map,/* output - mesh of isolines */
			double* 		ptr_mesh,/* input */
			unsigned int* 	ptr_Bnd,
			unsigned int 	size_row,
			unsigned int 	size_col,
			double 			V_1,/* value of range */
			double			V_2,
			unsigned int	index_range/* index of isoline */
			);
			

/*
* write picture of isolines to file ppm with given colour 
* - background colour is set
* - isolines colurs on input
* - colours of electrodes on input
*/
int
lbmn_anls_xy_D_fprintf_mesh_isolines_bw_netpbm_P3(
            const char*    	filename,
            unsigned int*  	ptr_Bnd,
			unsigned int*	ptr_mesh_isoline,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned char 	colour_background_red,/* colour - background */
			unsigned char 	colour_background_green,
			unsigned char 	colour_background_blue,
			unsigned char* 	ptr_isoline_colours_red,/* colours of isolines */
			unsigned char* 	ptr_isoline_colours_green,
			unsigned char* 	ptr_isoline_colours_blue,
			unsigned int 	number_of_isoline_colours,
			unsigned char* 	ptr_electrode_colours_red,/* colours of electrodes */
			unsigned char* 	ptr_electrode_colours_green,
			unsigned char* 	ptr_electrode_colours_blue);



/*
* write mesh (type double) to file ppm with given colour mapping function fptr_colormap
* it works with meshes: V, E_x, E_y, E1, E2
*/

int
lbmn_anls_xy_D_fprintf_mesh_isolines_colormap_netpbm_P3(
            const char*    filename,
            double*        ptr_mesh,
			unsigned int*  ptr_Bnd,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col,
            int(*fptr_colormap)(   
                unsigned char* ptr_red,
                unsigned char* ptr_green,
                unsigned char* ptr_blue,
                double val,
                double val_min,
                double val_max),
			double	V_min,/* range of displayed value */
			double	V_max,
			unsigned char colour_below_red,/* colour - below range */
			unsigned char colour_below_green,
			unsigned char colour_below_blue,
			unsigned char colour_abowe_red,/* colour - abowe range */
			unsigned char colour_abowe_green,
			unsigned char colour_abowe_blue,
			unsigned char* ptr_electrode_colours_red,
			unsigned char* ptr_electrode_colours_green,
			unsigned char* ptr_electrode_colours_blue,
			unsigned int number_of_colours);
			

/*
* determine Bnd, V, E_x, E_y at the given node (i_row, i_col)
*/	
int
lbmn_anls_xy_D_goto_ij(
				double*			ptr_pos_x,
				double*			ptr_pos_y,
				unsigned int*	ptr_Bnd,/*output*/
				double*			ptr_V,
				double*			ptr_E_x,
				double*			ptr_E_y,
				double*			ptr_mesh_V,/* input */
				unsigned int*	ptr_mesh_Bnd,
				unsigned int	size_row,
				unsigned int	size_col,
				double			x_min,
				double			x_max,
				double			y_min,
				double			y_max,
				unsigned int	i_row,
				unsigned int	i_col				
				);


/*
* determine Bnd, V, E_x, E_y at the given node (i_row, i_col)
*/	
int
lbmn_anls_xy_D_goto_xy(
				unsigned int*	ptr_pos_x,
				unsigned int*	ptr_pos_y,
				unsigned int*	ptr_Bnd,/*output*/
				double*			ptr_V,
				double*			ptr_E_x,
				double*			ptr_E_y,
				double*			ptr_mesh_V,/* input */
				unsigned int*	ptr_mesh_Bnd,
				unsigned int	size_row,
				unsigned int	size_col,
				double			x_min,/* mesh borders x_min, ... , x_max */
				double			x_max,
				double			y_min,
				double			y_max,
				double			pos_x,/* position of our point on mesh (x, y) [mm] */
				double			pos_y				
				);


#endif /* LBMN_ANLS_XY_D_H_ */

/* ==== end of file lbmn_anls_xy_D.h ==== */