/* lbmn_graphics_colormap.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "lbmn_colormap.h"

double
lbmn_colormap_clamp_0_1(double x)
{
	if(x < 0.0)
	{
		return 0.0;
	}
	else if(x > 1.0)
	{
		return 1.0;
	}
	else
	{
		/* 
		0.0 <= x <= 1.0 
		*/
		return x;
	}
	
}/* lbmn_colormap_clamp_0_1 */

/*=====================================================================*/

int
lbmn_colormap_hottocold_clamp(
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
	
	double A = 0.0;
	double B = 0.0;
	
	double t = 0.0;
	
	double t_r = 0.0;
	double t_g = 0.0;
	double t_b = 0.0;
	
	double tmp = 0.0;
	
	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/**/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	/* now val_max > val_min*/
	
	
	if(val < val_min)
    {/* val < val_min */

		/* set colour to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return -1;

    }/* val < val_min */
	else if(val > val_max)
    {/* val > val_max */

		/*  set colour to white */
        *ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return +1;
		
    }/* val > val_max */
    else
    {/* val_min <= val <= val_max */

		/*
		* val_min > val_max 
		*/
		A = 2.0 / (val_max - val_min);		
		B = 1.0 - A * val_max;
		
        t = A * val + B;
		
		/* red */
		tmp = 2.0 * t - 2.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_r = -tmp;
		}
		else
		{
			t_r = tmp;
		}
		
		/* green */
		tmp = 2.0 * t;
		
		if(tmp < 0.0)/* abs */
		{
			t_g = -tmp;
		}
		else
		{
			t_g = tmp;
		}
		
		/* blue */
		tmp = 2.0 * t + 2.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_b = -tmp;
		}
		else
		{
			t_b = tmp;
		}

		/* set colour */
        *ptr_red    = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_r));
        *ptr_green  = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_g));
        *ptr_blue   = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_b));

		/* success */
        return 0;

    }/* val_min <= val <= val_max */
	
}/* lbmn_colormap_hottocold_clamp */

/*===========================================================================*/

int
lbmn_colormap_hottocold_clamp_inv(
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
	
	double A = 0.0;
	double B = 0.0;
	
	double t = 0.0;
	
	double t_r = 0.0;
	double t_g = 0.0;
	double t_b = 0.0;
	
	double tmp = 0.0;
	
	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/**/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	/* now val_max > val_min*/
	
	
	if(val < val_min)
    {/* val < val_min */

		/* set colour  to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return -1;

    }/* val < val_min */
	else if(val > val_max)
    {/* val > val_max */

		/* set colour to white */
        *ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return +1;
		
    }/* val > val_max */
    else
    {/* val_min <= val <= val_max */

		/*
		* val_min > val_max 
		*/
	
	
		A = 2.0 / (val_min - val_max);		
		B = 1.0 - A * val_min;
	
        t = A * val + B;
		
		/* red */
		tmp = 2.0 * t - 2.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_r = -tmp;
		}
		else
		{
			t_r = tmp;
		}
		
		/* green */
		tmp = 2.0 * t;
		
		if(tmp < 0.0)/* abs */
		{
			t_g = -tmp;
		}
		else
		{
			t_g = tmp;
		}
		
		/* blue */
		tmp = 2.0 * t + 2.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_b = -tmp;
		}
		else
		{
			t_b = tmp;
		}

		/* set colour */
        *ptr_red    = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_r));
        *ptr_green  = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_g));
        *ptr_blue   = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(2.0 - t_b));

		/* success */
        return 0;
		
    }/* val_min <= val <= val_max */
	
}/* lbmn_colormap_hottocold_clamp_inv */
	
/*============================================*/

int
lbmn_colormap_jet_clamp(
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
	
	double A = 0.0;
	double B = 0.0;
	
	double t = 0.0;
	
	double t_r = 0.0;
	double t_g = 0.0;
	double t_b = 0.0;
	
	double tmp = 0.0;
	
	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/**/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	/* now val_max > val_min*/
	
	
	if(val < val_min)
    {/* val < val_min */

		/* set colout to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return -1;
		
    }/* val < val_min */
    else if(val > val_max)
    {/* val < val_min */
        
		/* set colout to white */
		*ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return +1;
		
    }/* val < val_min */
    else
    {/* val_min <= val <= val_max */

		/*
		* val_min > val_max 
		*/
		A = 2.0 / (val_max - val_min);		
		B = 1.0 - A * val_max;
		
        t = A * val + B;
		
		/* red */
		tmp = 2.0 * t - 1.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_r = -tmp;
		}
		else
		{
			t_r = tmp;
		}
		
		/* green */
		tmp = 2.0 * t;
		
		if(tmp < 0.0)/* abs */
		{
			t_g = -tmp;
		}
		else
		{
			t_g = tmp;
		}
		
		/* blue */
		tmp = 2.0 * t + 1.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_b = -tmp;
		}
		else
		{
			t_b = tmp;
		}

		/* set colour */
        *ptr_red    = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_r));
        *ptr_green  = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_g));
        *ptr_blue   = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_b));

		/* success */
        return 0;
		
    }/* val_min <= val <= val_max */		
	
}/* lbmn_colormap_jet_clamp */
	
	
/*===============================================================*/

int
lbmn_colormap_jet_clamp_inv(
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
	
	double A = 0.0;
	double B = 0.0;
	
	double t = 0.0;
	
	double t_r = 0.0;
	double t_g = 0.0;
	double t_b = 0.0;
	
	double tmp = 0.0;
	
	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/**/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	
	/* now val_max > val_min*/
	
	
	if(val < val_min)
    {/* val < val_min */

		/* set colour to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return -1;
    }/* val < val_min */
    else if(val > val_max)
    {/* val > val_max */

		/* set colour to white */
        *ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return +1;
		
    }/* val > val_max */
    else
    {/* val_min >= val >= val_max */

		/*
		* val_min > val_max 
		*/
		A = 2.0 / (val_min - val_max);		
		B = 1.0 - A * val_min;
		
        t = A * val + B;
		
		/* red */
		tmp = 2.0 * t - 1.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_r = -tmp;
		}
		else
		{
			t_r = tmp;
		}
		
		/* green */
		tmp = 2.0 * t;
		
		if(tmp < 0.0)/* abs */
		{
			t_g = -tmp;
		}
		else
		{
			t_g = tmp;
		}
		
		/* blue */
		tmp = 2.0 * t + 1.0;
		
		if(tmp < 0.0)/* abs */
		{
			t_b = -tmp;
		}
		else
		{
			t_b = tmp;
		}

		/* set colour */
        *ptr_red    = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_r));
        *ptr_green  = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_g));
        *ptr_blue   = (unsigned char)(255.0 * lbmn_colormap_clamp_0_1(1.5 - t_b));

		/* success */
        return 0;
		
    }/* val_min >= val >= val_max */
	
}/* lbmn_colormap_jet_clamp_inv */

/*==========================================================================*/
	
int
lbmn_colormap_grayscale(    
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
    unsigned char tmp = 0;


	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/* val_min >= val_max*/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	/* now val_max > val_min*/
	
	
    if(val < val_min)
    {/* val < val_min */

		/* set colour to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return -1;
		
    }/* val < val_min */
    else if(val > val_max)
    {/* val > val_max */

		/* set colour to white */
        *ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return +1;
		
    }/* val > val_max */
    else
    {/* val_min <= val <= val_max */

        tmp = (unsigned char)(255.0 * ((val - val_min)/(val_max - val_min)));

		/* set colour */
        *ptr_red    = tmp;
        *ptr_green  = tmp;
        *ptr_blue   = tmp;

		/* success */
        return 0;
		
    }/* val_min <= val <= val_max */

}/* lbmn_colormap_grayscale */

/*===============================================================*/

int
lbmn_colormap_grayscale_inv(   
    unsigned char* ptr_red,
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,
    double val_min,
    double val_max)
{
    unsigned char tmp = 0;

	/* val_max must be greater than val_min */
	if(val_min >= val_max)
	{/**/
		
		/* report error */
		return -2;
		
	}/* val_min >= val_max */
	
	/* now val_max > val_min*/
	
	
    if(val < val_min)
    {/* val < val_min */

		/* set colout to white*/
        *ptr_red    = 255;
        *ptr_green  = 255;
        *ptr_blue   = 255;

		/* report problem */
        return -1;
		
    }/* val < val_min */
    else if(val > val_max)
    {/* val > val_max */

		/* set colour to black */
        *ptr_red    = 0;
        *ptr_green  = 0;
        *ptr_blue   = 0;

		/* report problem */
        return +1;
		
    }/* val > val_max */
    else
    {/* val_min <= val <= val_max */

        tmp = (unsigned char)(255.0 * ((val_max - val)/(val_max - val_min)));

		/* set colour */
        *ptr_red    = tmp;
        *ptr_green  = tmp;
        *ptr_blue   = tmp;

		/* success */
        return 0;
		
    }/* val_min <= val <= val_max */
	
}/* lbmn_colormap_grayscale_inv */

/* ==== end of file lbmn_colormap.c ==== */