/* lbmn_graphics_colormap.h
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

/* 
* Colormaps are described in Liebmann module: doc_colormap (LaTeX)
*/

#ifndef COLORMAP_H_
#define COLORMAP_H_

/* function clamp */
double
lbmn_colormap_clamp_0_1(double x);


/*colormaps
* input: val_min, val_max, val
*        where:
*        1)     val_min < val_max
*        2)     val_min <= val
*        3)                val <= val_max
*
* output: 3 pointers to images (3 colors) red/green/blue (0..255) (unsigned char)
*/
int
lbmn_colormap_hottocold_clamp(
    unsigned char* ptr_red,/* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);


int
lbmn_colormap_hottocold_clamp_inv(
    unsigned char* ptr_red,/* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);
	
int
lbmn_colormap_jet_clamp(
    unsigned char* ptr_red, /* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);
	
int
lbmn_colormap_jet_clamp_inv(
    unsigned char* ptr_red,/* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);

/*
colormaps, which do not use clamp function
*/
int
lbmn_colormap_grayscale(    
    unsigned char* ptr_red,/* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);

int
lbmn_colormap_grayscale_inv(   
    unsigned char* ptr_red,/* outputs */
    unsigned char* ptr_green,
    unsigned char* ptr_blue,
    double val,/* inputs */
    double val_min,
    double val_max);
	
#endif /* COLORMAP_H_ */

/* ==== end of file lbmn_colormap.h ==== */
