/* lbmn_mesh_xy_D.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "lbmn_mesh_xy_D.h"

/*
* try to allocate problem
*/
int
lbmn_mesh_xy_problem_allocate(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b,
			
            unsigned int size_row,/* input */
            unsigned int size_col,
            double x_start,
            double x_stop,
            double y_start,
            double y_stop)
{
	if(size_row < 3 ||
	   size_col < 3
	)
	{
		/* incorect size */
		return -1;
	}

    if(x_start >= x_stop)
    {
        /* incorrect x values */
        return -2;
    }
	
    if(y_start >= y_stop)
    {
        /* incorrect x values */
        return -3;
    }
	/* size seems to be correct */
	
	if(
		   *ptrptr_V   != NULL
		|| *ptrptr_Bnd != NULL
		|| *ptrptr_E_x != NULL
		|| *ptrptr_E_y != NULL
		|| *ptrptr_E_1 != NULL
		|| *ptrptr_E_2 != NULL
        || *ptrptr_isolines_V   != NULL
        || *ptrptr_isolines_E_x != NULL
        || *ptrptr_isolines_E_y != NULL
        || *ptrptr_isolines_E_1 != NULL
        || *ptrptr_isolines_E_2 != NULL
		|| *ptrptr_workbench_colours_r != NULL
		|| *ptrptr_workbench_colours_g != NULL
		|| *ptrptr_workbench_colours_b != NULL
		|| *ptrptr_workbench_bw_colours_r != NULL
		|| *ptrptr_workbench_bw_colours_g != NULL
		|| *ptrptr_workbench_bw_colours_b != NULL
	)
	{/* not all pointers are NULL */
	
		return -4;
		
	}/* not all pointers are NULL */
	else
	{/* all target pointers are NULL */

		/* try to allocate memory */

		*ptrptr_V   = (double*)       malloc(sizeof(double)      * size_row * size_col);
        *ptrptr_Bnd = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);
		*ptrptr_E_x = (double*)       malloc(sizeof(double)      * size_row * size_col);
		*ptrptr_E_y = (double*)       malloc(sizeof(double)      * size_row * size_col);
		*ptrptr_E_1 = (double*)       malloc(sizeof(double)      * size_row * size_col);
		*ptrptr_E_2 = (double*)       malloc(sizeof(double)      * size_row * size_col);
		
        *ptrptr_isolines_V   = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);
        *ptrptr_isolines_E_x = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);
        *ptrptr_isolines_E_y = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);
        *ptrptr_isolines_E_1 = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);
        *ptrptr_isolines_E_2 = (unsigned int*) malloc(sizeof(unsigned int)* size_row * size_col);

		*ptrptr_workbench_colours_r = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		*ptrptr_workbench_colours_g = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		*ptrptr_workbench_colours_b = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		
		*ptrptr_workbench_bw_colours_r = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		*ptrptr_workbench_bw_colours_g = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		*ptrptr_workbench_bw_colours_b = (unsigned char*)malloc(sizeof(double) * size_row * size_col);
		
		if(
			   *ptrptr_V   == NULL	
			|| *ptrptr_Bnd == NULL
			|| *ptrptr_E_x == NULL
			|| *ptrptr_E_y == NULL
			|| *ptrptr_E_1 == NULL
			|| *ptrptr_E_2 == NULL
            || *ptrptr_isolines_V   == NULL 
            || *ptrptr_isolines_E_x == NULL 
            || *ptrptr_isolines_E_y == NULL 
            || *ptrptr_isolines_E_1 == NULL 
            || *ptrptr_isolines_E_2 == NULL
			|| *ptrptr_workbench_colours_r == NULL
			|| *ptrptr_workbench_colours_g == NULL
			|| *ptrptr_workbench_colours_b == NULL
			|| *ptrptr_workbench_bw_colours_r == NULL
			|| *ptrptr_workbench_bw_colours_g == NULL
			|| *ptrptr_workbench_bw_colours_b == NULL
		)
		{/* not all the memory has been allocated */
		
			/* try to deallocate all the allocated resources */
			if(*ptrptr_V != NULL)
			{
				free(*ptrptr_V);
				*ptrptr_V = NULL;
				
			}
				
			if(*ptrptr_Bnd != NULL)
			{
				free(*ptrptr_Bnd);
				*ptrptr_Bnd = NULL;
			}
			
			if(*ptrptr_E_x != NULL)
			{
				free(*ptrptr_E_x);
				*ptrptr_E_x = NULL;
			}
			
			if(*ptrptr_E_y != NULL)
			{
				free(*ptrptr_E_y);
				*ptrptr_E_y = NULL;
			}
			
			if(*ptrptr_E_1 != NULL)
			{
				free(*ptrptr_E_1);
				*ptrptr_E_1 = NULL;
			}
			
			if(*ptrptr_E_2 != NULL)
			{
				free(*ptrptr_E_2);
				*ptrptr_E_2 = NULL;
			}
			
            if(*ptrptr_isolines_V   != NULL)
            {
                free(*ptrptr_isolines_V);
				*ptrptr_isolines_V = NULL;
            }
 
            if(*ptrptr_isolines_E_x != NULL)
            {
                free(*ptrptr_isolines_E_x);
				*ptrptr_isolines_E_x = NULL;
            }

            if(*ptrptr_isolines_E_y != NULL)
            {
                free(*ptrptr_isolines_E_y);
				*ptrptr_isolines_E_y = NULL;
            }

            if(*ptrptr_isolines_E_1 != NULL)
            {
                free(*ptrptr_isolines_E_1);
				*ptrptr_isolines_E_1 = NULL;
            }
 
            if(*ptrptr_isolines_E_2 != NULL)
            {
                free(*ptrptr_isolines_E_2);
				*ptrptr_isolines_E_2 = NULL;
            }


			if(*ptrptr_workbench_colours_r != NULL)
			{
				free(*ptrptr_workbench_colours_r);
				*ptrptr_workbench_colours_r = NULL;
			}
			
			if(*ptrptr_workbench_colours_g != NULL)
			{
				free(*ptrptr_workbench_colours_g);
				*ptrptr_workbench_colours_g = NULL;
			}
			
			if(*ptrptr_workbench_colours_b != NULL)
			{
				free(*ptrptr_workbench_colours_b);
				*ptrptr_workbench_colours_b = NULL;
			}
			
			if(*ptrptr_workbench_bw_colours_r != NULL)
			{
				free(*ptrptr_workbench_bw_colours_r);
				*ptrptr_workbench_bw_colours_r = NULL;
			}
			
			if(*ptrptr_workbench_bw_colours_g != NULL)
			{
				free(*ptrptr_workbench_bw_colours_g);
				*ptrptr_workbench_bw_colours_g = NULL;
			}
			
			if(*ptrptr_workbench_bw_colours_b != NULL)
			{
				free(*ptrptr_workbench_bw_colours_b);
				*ptrptr_workbench_bw_colours_b = NULL;
			}
			
			/* memory allocation failed */
			return -5;
			
		}/* not all the memory has been allocated */
		else
		{/* all memory allocated correctly */

            *ptr_size_row   = size_row;
            *ptr_size_col   = size_col;
            *ptr_x_start    = x_start;
            *ptr_x_stop     = x_stop;
            *ptr_y_start    = y_start;
            *ptr_y_stop     = y_stop;

			/* OK
			* return 0 at the end
			*/
			
		}/* all memory allocated correctly */

	}/* all target pointers are NULL */
	
	return 0;
	
}/* lbmn_mesh_xy_problem_allocate */

/*==========================================================*/

/*
* try to deallocate problem
*/
int
lbmn_mesh_xy_problem_destroy(
		    unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b)
{
    if(
        ptr_size_row    == NULL ||
        ptr_size_col    == NULL ||
        ptr_x_start     == NULL ||
        ptr_x_stop      == NULL ||
        ptr_y_start     == NULL ||
        ptr_y_stop      == NULL ||
		
		*ptrptr_V           == NULL ||
		*ptrptr_Bnd         == NULL ||
		*ptrptr_E_x         == NULL ||
		*ptrptr_E_y         == NULL ||
		*ptrptr_E_1         == NULL ||
		*ptrptr_E_2         == NULL ||
        *ptrptr_isolines_V  == NULL ||
        *ptrptr_isolines_E_x    == NULL ||
        *ptrptr_isolines_E_y    == NULL ||
        *ptrptr_isolines_E_1    == NULL ||
        *ptrptr_isolines_E_2    == NULL ||
		*ptrptr_workbench_colours_r == NULL ||
		*ptrptr_workbench_colours_g == NULL ||
		*ptrptr_workbench_colours_b == NULL ||
		*ptrptr_workbench_bw_colours_r  == NULL ||
		*ptrptr_workbench_bw_colours_g  == NULL ||
		*ptrptr_workbench_bw_colours_b == NULL
	)
	{/* some pointers are NULL */
		
	    /* memory allocation failure */
	    return -1;
			
	}/* some pointers are NULL */
	else
	{/* all the pointers are not NULL */
			

        *ptr_size_row    = 0;
        *ptr_size_col    = 0;

        *ptr_x_start     = 0.0;
        *ptr_x_stop      = 0.0;
        *ptr_y_start     = 0.0;
        *ptr_y_stop      = 0.0;

		/* deallocate memory */

		free(*ptrptr_V);
		*ptrptr_V = NULL;
		
		free(*ptrptr_Bnd);
		*ptrptr_Bnd = NULL;
		
		free(*ptrptr_E_x);
		*ptrptr_E_x = NULL;
		
		free(*ptrptr_E_y);
		*ptrptr_E_y = NULL;
		
		free(*ptrptr_E_1);
		*ptrptr_E_1 = NULL;
		
		free(*ptrptr_E_2);
		*ptrptr_E_2 = NULL;
		
        free(*ptrptr_isolines_V);
		*ptrptr_isolines_V = NULL;
		
        free(*ptrptr_isolines_E_x);
		*ptrptr_isolines_E_x = NULL;
		
        free(*ptrptr_isolines_E_y);
		*ptrptr_isolines_E_y = NULL;
		
        free(*ptrptr_isolines_E_1);
		*ptrptr_isolines_E_1 = NULL;
		
        free(*ptrptr_isolines_E_2);
		*ptrptr_isolines_E_2 = NULL;
		
		free(*ptrptr_workbench_colours_r);
		*ptrptr_workbench_colours_r = NULL;
		
		free(*ptrptr_workbench_colours_g);
		*ptrptr_workbench_colours_g = NULL;
		
		free(*ptrptr_workbench_colours_b);
		*ptrptr_workbench_colours_b = NULL;
		
		free(*ptrptr_workbench_bw_colours_r);
		*ptrptr_workbench_bw_colours_r = NULL;
		
		free(*ptrptr_workbench_bw_colours_g);
		*ptrptr_workbench_bw_colours_g = NULL;
		
		free(*ptrptr_workbench_bw_colours_b);
		*ptrptr_workbench_bw_colours_b = NULL;
			
		/* return 0 at the end of function */
			
	}/* all the pointers are not NULL */
		
	return 0;
	
}/* lbmn_mesh_xy_problem_destroy */


/*===================================================================*/


int
lbmn_mesh_xy_D_set_point_ij(
                    double* 		ptr_V,
                    unsigned int* 	ptr_Bnd,
                    unsigned int   	size_row,/* size of mesh */
                    unsigned int   	size_col,
                    unsigned int   	i_row,/* point position (1,2, .. , n_row)*/
                    unsigned int   	i_col,/* 1,2,..,n_col*/
                    double 			V  , /*input point values*/
					unsigned int    boundary)
{
	if( ptr_V   == NULL ||
		ptr_Bnd == NULL ||
		size_row < 3 ||
		size_col < 3 
	)
	{
		return -1;
	}
	
    if(i_row > size_row)
    {
        return -2;
    }
    
    if(i_col > size_col)
    {
        return -3;
    }

    /* set mesh point parameters */
    *(ptr_V   +   (i_row - 1) * size_col + i_col - 1) = V;
	*(ptr_Bnd +   (i_row - 1) * size_col + i_col - 1) = boundary;
	
    return 0;
	
}/* lbmn_mesh_xy_D_set_point_ij */


/*=====================================================================*/

int
lbmn_mesh_xy_D_set_rectangle_ij(
                    double*			ptr_V,
                    unsigned int*   ptr_Bnd,
                    unsigned int    size_row,/* size of mesh */
                    unsigned int    size_col,
                    unsigned int    i_row_start,/* rectangle parameters */
					unsigned int    i_row_stop,
					unsigned int    i_col_start,
                    unsigned int    i_col_stop,
                    double			V, /*input rectangle values*/
					unsigned int    boundary)
{
	/* declartion of variables */
	
	unsigned int i_row = 0;
	unsigned int i_col = 0;
	
	int error_code = 0;
	
	/* instructions */
	
	/* validate input parameters */
	
	if(	ptr_V   == NULL ||
		ptr_Bnd == NULL ||
		size_row < 3    ||
		size_col < 3
	)
	{
		return -1;
	}
	
	if(	i_row_start > size_row ||
		i_row_stop  > size_row)
	{
		return -2;
	}
	
	
	if(	i_col_start > size_col ||
		i_col_stop  > size_col)
	{
		return -3;
	}

	if(i_row_start > i_row_stop)
	{
		return -4;
	}
	
	if(i_col_start > i_col_stop)
	{
		return -5;
	}
	
	/* now input data seems to be correct */
	
	for(i_row = i_row_start; i_row <= i_row_stop; i_row++)
	{
		for(i_col = i_col_start; i_col <= i_col_stop; i_col++)
		{
			error_code = lbmn_mesh_xy_D_set_point_ij(
							ptr_V,
							ptr_Bnd,							
							size_row,/* size of picture */
							size_col,
							i_row,/* point position (1,2, .. , n_row)*/
							i_col,/* 1,2,..,n_col*/
							V, /*input node values*/
							boundary);
							
			if(error_code != 0)
			{
				return -6;
			}
			
		}/*loop i_col*/
		
	}/* loop i_row */
	
	/* success */
	return 0;
	
}/* lbmn_mesh_xy_D_set_rectange_ij */

/*=======================================================*/

/*
*
* displaying a mesh on display (x,y)
*/

int
lbmn_mesh_xy_D_printf_V(
            double*        ptr_V,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col)
{
    
    /* declaion of variables */

    unsigned int i_row = 0;
    unsigned int i_col = 0;

    double tmp = 0.0;

    /* instructions */

    for(i_row = size_row; i_row > 0; i_row--)
    {
        for(i_col = 1; i_col <= size_col; i_col++)
        {
            tmp = *(ptr_V + (i_row-1) * size_col + i_col - 1);
            printf("%+18.15E", tmp);

        }/*i_col loop*/
        
        printf("\n");

    }/*i_row loop*/

	/* end */
	return 0;
	
}/* lbmn_mesh_xy_D_printf_V */


/*===============================================================*/

/*
*
* displaying boundaries (x,y)
*/

int
lbmn_mesh_xy_D_printf_Bnd(
            unsigned int*   ptr_Bnd,
            unsigned int    size_row,/* size of mesh */
            unsigned int    size_col)
{
    /* declaration of variables */
	
	unsigned int i_row = 0;
    unsigned int i_col = 0;

    unsigned int tmp = 0.0;

	/* instructions */
	
    /* without axis */
    for(i_row = size_row; i_row > 0; i_row--)
    {
        for(i_col = 1; i_col <= size_col; i_col++)
        {
            tmp = *(ptr_Bnd + (i_row-1) * size_col + i_col - 1);
            printf("%d ", tmp);

        }/*i_col loop*/
        
        printf("\n");

    }/*i_row loop*/

	/* end */
	return 0;
	
}/* lbmn_mesh_xy_D_printf_Bnd */

/*===============================================================*/

int
lbmn_mesh_xy_D_find_V_min_max(
                        double*     ptr_min,
                        double*     ptr_max,
                        double*     ptr_V,  
                        unsigned int size_row,/* size of mesh */
                        unsigned int size_col)
{
	/* declaration of variables */

    double tmp_min = 0.0;
    double tmp_max = 0.0;

    unsigned int i_row = 0;
    unsigned int i_col = 0;

    double tmp = 0;

	/* instructions */

    /* find min / max */
    tmp_min = *(ptr_V + 0);
    tmp_max = *(ptr_V + 0);

    for(i_row = 1; i_row <= size_row; i_row++)
    {

        for(i_col = 1; i_col <= size_col; i_col++)
        {
            
            tmp = *(ptr_V + (i_row-1) * size_col + i_col - 1);

            if(tmp < tmp_min)
            {
                tmp_min = tmp;
            }

            if(tmp > tmp_max)
            {
                tmp_max = tmp;
            }

        }/* i_col loop*/

    }/* i_col loop */

    /* end of searching for min / max*/

    *ptr_min = tmp_min;
    *ptr_max = tmp_max;

    return 0;

}/* lbmn_relax_xy_D_find_V_min_max */


/*===================================================*/


int
lbmn_mesh_xy_D_fprintf_mesh_double_netpbm_P3(
            const char* 	filename,
            double*        	ptr_mesh,
			unsigned int* 	ptr_Bnd,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
            int(*fptr_colormap)(   
                unsigned char* ptr_red,
                unsigned char* ptr_green,
                unsigned char* ptr_blue,
                double val,
                double val_min,
                double val_max),
			double	V_min,
			double	V_max,
			unsigned char colour_below_red,/* colour - below range */
			unsigned char colour_below_green,
			unsigned char colour_below_blue,
			unsigned char colour_abowe_red,/* colour - abowe range */
			unsigned char colour_abowe_green,
			unsigned char colour_abowe_blue,
			unsigned char* 	ptr_electrode_colours_red,
			unsigned char* 	ptr_electrode_colours_green,
			unsigned char* 	ptr_electrode_colours_blue,
			unsigned int 	number_of_colours)
{
    
    /* declaration of variables */

    unsigned int i_row = 0;
    unsigned int i_col = 0;

	unsigned int boundary = 0;
	
	unsigned int icp = 0;
	
	unsigned char tmp_red;
	unsigned char tmp_green;
	unsigned char tmp_blue;

	double value = 0.0;/* value of mesh point ( for example V(row,col) )*/

    FILE* ptr_file = NULL;


    /* instructions */

	/* input data errors */
	
	if(filename == NULL)
	{
		return -1;
	}
	
	if(ptr_mesh == NULL)
	{
		return -2;
	}

	if(ptr_Bnd == NULL)
	{
		return -3;
	}
	
	
	if(size_row < 3)
	{
		return -4;
	}
	
	if(size_col < 3)
	{
		return -5;
	}
	
	if(fptr_colormap == NULL)
	{
		return -6;
	}
	
	
	if(	ptr_electrode_colours_red 	== NULL ||
		ptr_electrode_colours_green == NULL ||
		ptr_electrode_colours_blue 	== NULL
	)
	{
		return -7;
	}
	
	if(number_of_colours < 1)
	{
		return -8;
	}
	
	/* if no errors - try to do open file */
	
	ptr_file = fopen (filename,"w");

    if (ptr_file == NULL)
    {
        /* could not open output file */
        return -9;
    }
    else
    {/*opened output file */

        /* write file header */

        fprintf (ptr_file, "P3\n %u %u\n 255\n", size_col, size_row);
 
        if(ferror(ptr_file))
        {
            /* could not write header */
            fclose(ptr_file);
            return -10;
        }
        else
        {/* header written successfully */
            
            
            for(i_row = size_row; i_row > 0; i_row--)
            {
                for(i_col = 1; i_col <= size_col; i_col++)
                {

					/* determine index of central point */
					icp = (i_row-1) * size_col + i_col -1;
					
					/* determine boundary value*/
					boundary = *(ptr_Bnd + icp);
									
									
					if(boundary > 0)
					{/* mesh point is electrode */
				
						fprintf(ptr_file, "%3u %3u %3u\n",
									*(ptr_electrode_colours_red   + boundary),
									*(ptr_electrode_colours_green + boundary),
									*(ptr_electrode_colours_blue  + boundary));
						
						if(ferror(ptr_file))
						{
							/* could not write data to file (electrode) */
							fclose(ptr_file);
							return -11;
						}
				
					}/* mesh point is electrode */
					else
					{/* mesh point is vacuum */
				
						value = *(ptr_mesh + icp);
						
						if(value < V_min)
						{/* value below range */
					
							/* try to write data to file */
							fprintf(ptr_file, "%3u %3u %3u\n",	colour_below_red,
																colour_below_green,
																colour_below_blue);

							if(ferror(ptr_file))
							{
								fclose(ptr_file);
								return -12;
							}
							
						}/* value below range */
						else if(value > V_max)
						{/* value abowe range */
					
							/* try to write data to file */
							fprintf(ptr_file, "%3u %3u %3u\n",	colour_abowe_red,
																colour_abowe_green,
																colour_abowe_blue);

							if(ferror(ptr_file))
							{
								fclose(ptr_file);
								return -13;
							}					
						}/* value abowe range */
						else
						{/* value in range V_min ... V_max*/
							
							if(fptr_colormap(    
										&tmp_red,
										&tmp_green,
										&tmp_blue,
										*(ptr_mesh + (i_row-1) * size_col + i_col - 1),
										V_min,
										V_max))
							{/* probem with mapping value to colour */
							
								fclose(ptr_file);
								return -14;
								
							}/* problem with mappig value to colour */
							else
							{/*colormap ok*/  
			  
								fprintf(ptr_file, "%3u %3u %3u\n",tmp_red, tmp_green, tmp_blue);

								if(ferror(ptr_file))
								{
									fclose(ptr_file);
									return -15 ;
								}
							}/*colormap ok*/
							
						}/* value in range V_min ... V_max */
						
					}/* mesh point is vacuum */

                }/* loop i_col*/

            }/* loop i_row*/

            /* success */
            fclose(ptr_file);
                
        }/* header written successfully */
        
    }/* opened output file */

    return 0;

}/* lbmn_mesh_xy_D_fprintf_mesh_double_netpbm_P3 */


/*==================================================================*/


int
lbmn_mesh_xy_D_fprintf_mesh_double_dat(
            const char* 	filename,
            double*        	ptr_mesh,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col)
{
    
    /* declaration of variables */

    unsigned int i_row = 0;
    unsigned int i_col = 0;

    FILE* ptr_file = NULL;

    /* instructions */

    ptr_file = fopen (filename,"w");

    if (ptr_file == NULL)
    {
        /* could not open output file */
        return -1;
    }
    else
    {/*opened output file */
         

        for(i_row = size_row; i_row > 0; i_row--)
        {
            /*try to write another line*/
            for(i_col = 1; i_col <= size_col; i_col++)
            {

                /* try to write another mesh point to file */

                fprintf(ptr_file, "%+18.15E ", *(ptr_mesh + (i_row-1) * size_col + i_col - 1));

                if(ferror(ptr_file))
                {
                    /* error */
                    fclose(ptr_file);
                    return -3;
                }
            }/*i_col*/

            /* try to write end of line */

            fprintf(ptr_file, "\n");
            if(ferror(ptr_file))
            {
                /* error */
                fclose(ptr_file);
                return -4;
            }

        }/*i_row loop*/


        /* success (if no errors) */
        fclose(ptr_file);

        
    }/* opened output file */

    return 0;

}/* lbmn_mesh_xy_D_fprintf_mesh_double_dat */


/*===================================================================*/


int
lbmn_mesh_xy_D_fprintf_mesh_uint_dat(
            const char*     filename,
            unsigned int*   ptr_mesh,
            unsigned int    size_row,/* size of mesh */
            unsigned int    size_col)
{
    
    /* declaration of variables */

    unsigned int i_row = 0;
    unsigned int i_col = 0;

    FILE* ptr_file = NULL;

    /* instructions */

    ptr_file = fopen (filename,"w");

    if (ptr_file == NULL)
    {
        /* could not open output file */
        return -1;
    }
    else
    {/*opened output file */

         

        for(i_row = size_row; i_row > 0; i_row--)
        {
            /* try to write another row */
            for(i_col = 1; i_col <= size_col; i_col++)
            {

                /* try to write another mesh point to file */
                fprintf(ptr_file, "%u ", *(ptr_mesh + (i_row-1) * size_col + i_col - 1));

                if(ferror(ptr_file))
                {
                    /* error */
                    fclose(ptr_file);
                    return -3;
                }
            }/*i_col loop*/

            
            /* try to write end of line to file */
            fprintf(ptr_file, "\n");

            if(ferror(ptr_file))
            {
                /* error */
                fclose(ptr_file);
                return -4;
            }

        }/*i_row* loop */


        /* success (if no errors) */
        fclose(ptr_file);

        
    }/* opened output file */

    return 0;

}/* lbmn_mesh_xy_D_fprintf_mesh_uint_dat */

/*====================================================================*/

int
lbmn_mesh_xy_D_create_image_workbench_bw(
            unsigned char* 	ptr_red,
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd)/* mesh of boundaries*/
{
	/* declaration of variables */

	unsigned int i_row;
	unsigned int i_col;

	unsigned int point_Bnd = 0;

	/* instructions */
	
	if(
		ptr_red 	== NULL ||
		ptr_green	== NULL ||
		ptr_blue	== NULL ||
		ptr_Bnd 	== NULL ||
		size_row < 3   		||
		size_col < 3		
	  )
	{
		return -1;
	}
	
	
		
	for(i_row = 1; i_row <= size_row; i_row++)
	{
		for(i_col = 1; i_col <= size_col; i_col++)
		{
			/* determine electrode point */
			point_Bnd =  *(ptr_Bnd + (i_row-1)* size_col + i_col - 1);
			
			
			if(point_Bnd != 0)
			{/*electrode point*/
		
				/* set colour to black */
				*(ptr_red 	+ (i_row-1)* size_col + i_col - 1) = 0;
				*(ptr_green + (i_row-1)* size_col + i_col - 1) = 0;
				*(ptr_blue	+ (i_row-1)* size_col + i_col - 1) = 0;
				
				
			}/* electrode point*/
			
			else
			{/*vacuum*/
		
				/* set colour to white */				
				*(ptr_red 	+ (i_row-1)* size_col + i_col - 1) = 255;
				*(ptr_green + (i_row-1)* size_col + i_col - 1) = 255;
				*(ptr_blue	+ (i_row-1)* size_col + i_col - 1) = 255;

			}/*vacuum*/
			
		}/* loop i_col */
		
	}/* loop i_row */

	return 0;
	
}/* lbmn_relax_zr_D_create_image_workbench_bw */

/*==============================================================================*/

int
lbmn_mesh_xy_D_create_image_workbench_colours(
            unsigned char* 	ptr_red,
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd,/* mesh of boundaries*/
			unsigned char*	ptr_electrode_colours_red,
			unsigned char*	ptr_electrode_colours_green,
			unsigned char*	ptr_electrode_colours_blue,
			unsigned int	size_electrode_colours)
{
	/* declaration of variables */
	
	unsigned int i_row;
	unsigned int i_col;
	
	unsigned int point_Bnd = 0;

	/* instructions */
	
	if(
		ptr_red 	== NULL ||
		ptr_green	== NULL ||
		ptr_blue	== NULL ||
		ptr_Bnd 	== NULL ||
		size_row < 3   		||
		size_col < 3		||
		ptr_electrode_colours_red 	== NULL ||
		ptr_electrode_colours_green == NULL ||
		ptr_electrode_colours_blue 	== NULL ||
		size_electrode_colours == 0
		
	  )
	{

        printf("ptr_red   == %p\n", ptr_red);
        printf("ptr_green == %p\n", ptr_green);
        printf("ptr_blue  == %p\n", ptr_blue);
        printf("size_row  == %u\n", size_row);
        printf("size_col  == %u\n", size_col);
        printf("ptr_Bnd   == %p\n", (void*)ptr_Bnd);
        printf("ptr_electrode_colours_red   == %p\n", ptr_electrode_colours_red);
        printf("ptr_electrode_colours_green == %p\n", ptr_electrode_colours_green);
        printf("ptr_electrode_colours_blue  == %p\n", ptr_electrode_colours_blue);
        printf("size_electrode_colours == %u\n", size_electrode_colours);

		return -1;
	}
	
	
	for(i_row = 1; i_row <= size_row; i_row++)
	{
		for(i_col = 1; i_col <= size_col; i_col++)
		{
			/* determine electrode point */
			point_Bnd =  *(ptr_Bnd + (i_row-1)* size_col + i_col - 1);
			
			
			/* set colour */

			*(ptr_red 	+ (i_row-1)* size_col + i_col - 1) = *(ptr_electrode_colours_red   + point_Bnd);
			*(ptr_green + (i_row-1)* size_col + i_col - 1) = *(ptr_electrode_colours_green + point_Bnd);
			*(ptr_blue	+ (i_row-1)* size_col + i_col - 1) = *(ptr_electrode_colours_blue  + point_Bnd);
		
		}/* loop i_col */
		
	}/* loop i_row */

	return 0;
	
}/* lbmn_relax_zr_D_create_image_workbench_colours */

/*====================================================================*/\

int
lbmn_mesh_xy_D_fprintf_image_workbench_netpbm_P3(
            const char*    filename,
			unsigned char* 	ptr_red,/* 3 input r/g/b channels */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,  
            unsigned int   size_row,/* size of these r/g/b channels */
            unsigned int   size_col)
{
	
	/* declaration of variables */

    unsigned int i_row = 0;
    unsigned int i_col = 0;

    FILE* ptr_file = NULL;


    /* instructions */

    ptr_file = fopen (filename,"w");

    if (ptr_file == NULL)
    {
        /* could not open output file */
        return -1;
    }
    else
    {/*opened output file */

        /* write file header */

        fprintf (ptr_file, "P3\n %u %u\n 255\n", size_col, size_row);
 
        if(ferror(ptr_file))
        {
            /* could not write header */
            fclose(ptr_file);
            return -2;
        }
        else
        {/* header written successfully */
            
              
            for(i_row = size_row; i_row >= 1; i_row--)
            {
                for(i_col = 1; i_col <= size_col; i_col++)
                {

					/* tru to write another pixel to PBM P3 file */
					
					fprintf(ptr_file, "%3u %3u %3u\n",
									*(ptr_red   + (i_row-1)* size_col + i_col - 1),
									*(ptr_green + (i_row-1)* size_col + i_col - 1),
									*(ptr_blue  + (i_row-1)* size_col + i_col - 1));

                    if(ferror(ptr_file))
                    {
						/* error */
						fclose(ptr_file);
						
                        return -4;
						
					}/* error */                    

                }/*loop i_col*/

            }/* loop i_row*/



            /* success */
            fclose(ptr_file);
                
        }/* header written successfully */
        
    }/* opened output file */
	
	return 0;
	
}/* lbmn_mesh_xy_D_fprintf_image_workbench_netpbm_P3 */


/* ================================================================== */


int
lbmn_mesh_xy_D_calculate_tab_d1V_dx1(
            double*     	ptr_d1V_dx1,
			double*			ptr_V,
			unsigned int    size_row,
            unsigned int    size_col,
			double 			mesh_step_x)
{
	/* declaration of variables */

	unsigned int index_row = 0;
	unsigned int index_col = 0;
	
	
	/* instructions */
	
	/* check correctness if input parameters */
	
	if(ptr_d1V_dx1 == NULL)
	{
		return -1;
	}
	
	if(ptr_V == NULL)
	{
		return -2;
	}
	
	if(size_row < 3)
	{
		return -3;
	}
	
	if(size_col < 3)
	{
		return -4;
	}
	
	if(mesh_step_x <= 0.0)
	{
		return -5;
	}
	
	/* else input parameters seem to be correct */
	
	/*traverse the tab_V matrix */
	
	/* P1 - 1st row, left corner */
	*(ptr_d1V_dx1 + 0) =  0.5 * ( *(ptr_V + 1) - *(ptr_V + 0)) / mesh_step_x; 
	
	
	/*  P2 - 1st row */
	for(index_col = 2; index_col <= size_col - 1; index_col ++)
	{
		
		*(ptr_d1V_dx1 + index_col - 1) =  0.5 * ( 
												*(ptr_V + index_col - 1 + 1) 
												-
												*(ptr_V + index_col - 1 - 1))
												/ mesh_step_x;
		
	}/* P2 - 1sr row*/
	
	
	/* P3 - 1st row, right corner */
	*(ptr_d1V_dx1 + size_col - 1) =  0.5 * ( 
											*(ptr_V + size_col - 1) 
											-
											*(ptr_V + size_col - 1 - 1)) 
											/ mesh_step_x; 
	
	/* P4 - P5 - P6 */
	for(index_row = 2; index_row <= size_row - 1; index_row ++)
	{
		/* P4 */
		*(ptr_d1V_dx1 + size_col * (index_row - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + 1)
															  -
															  *(ptr_V + size_col * (index_row - 1) + 0 )
															  ) 
															  / mesh_step_x;
		
		/* P5 */
		for(index_col = 2; index_col <= size_col - 1 ; index_col ++)
		{
		
			*(ptr_d1V_dx1 + size_col * (index_row - 1) + (index_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + (index_col - 1) + 1)
															  -
															  *(ptr_V + size_col * (index_row - 1) + (index_col - 1) - 1)
															  ) 
															  / mesh_step_x;
			
		}/* P5*/
		
		/* P6 */
		*(ptr_d1V_dx1 + size_col * (index_row - 1) + (size_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + (size_col - 1) + 0)
															  -
															  *(ptr_V + size_col * (index_row - 1) + (size_col - 1) - 1)
															  ) 
															  / mesh_step_x;

	}/* P4 - P5 - P6 */
	
	
	
	
	
	/* P7 */
	*(ptr_d1V_dx1 + size_col * (size_row - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + 1)
															  -
															  *(ptr_V + size_col * (size_row - 1) + 0 )
															  ) 
															  / mesh_step_x;


	/* P8 */
	for(index_col = 2; index_col <= size_col - 1; index_col ++)
	{
		*(ptr_d1V_dx1 + size_col * (size_row - 1) + (index_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + (index_col - 1) + 1)
															  -
															  *(ptr_V + size_col * (size_row - 1) + (index_col - 1) - 1)
															  ) 
															  / mesh_step_x;

	}


	/* P9 */
	*(ptr_d1V_dx1 + size_col * (size_row - 1) + size_col - 1) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + size_col - 1 + 0)
															  -
															  *(ptr_V + size_col * (size_row - 1) + size_col - 1 - 1)
															  ) 
															  / mesh_step_x;

	
	/* data processed succcessfully */
	return 0;
	
}/* lbmn_mesh_xy_D_calculate_tab_d1V_dx1()*/


/* =================================================================== */


int
lbmn_mesh_xy_D_calculate_tab_d1V_dy1(
            double*     	ptr_d1V_dy1,
			double*			ptr_V,
			unsigned int    size_row,
            unsigned int    size_col,
			double 			mesh_step_y)
{
	/* declaration of variables */

	unsigned int index_row = 0;
	unsigned int index_col = 0;
	
	
	/* instructions */
	
	/* check correctness if input parameters */
	
	if(ptr_d1V_dy1 == NULL)
	{
		return -1;
	}
	
	if(ptr_V == NULL)
	{
		return -2;
	}
	
	if(size_row < 3)
	{
		return -3;
	}
	
	if(size_col < 3)
	{
		return -4;
	}
	
	if(mesh_step_y <= 0.0)
	{
		return -5;
	}
	
	/* else input parameters seem to be correct */
	
	/*traverse the tab_V matrix */
	
	/* P1 - 1st row, left corner */
	*(ptr_d1V_dy1 + 0) =  0.5 * ( *(ptr_V + size_col) - *(ptr_V + 0)) / mesh_step_y; 
	
	
	/*  P2 - 1st row */
	for(index_col = 2; index_col <= size_col - 1; index_col ++)
	{
		
		*(ptr_d1V_dy1 + index_col - 1) =  0.5 * ( 
												*(ptr_V + index_col - 1 + size_col) 
												-
												*(ptr_V + index_col - 1 + 0))
												/ mesh_step_y;
		
	}/* P2 - 1st row*/
	
	
	/* P3 - 1st row, right corner */
	*(ptr_d1V_dy1 + size_col - 1) =  0.5 * ( 
											*(ptr_V + size_col - 1 + size_col) 
											-
											*(ptr_V + size_col - 1 + 0))
											/ mesh_step_y; 
	
	/* P4 - P5 - P6 */
	for(index_row = 2; index_row <= size_row - 1; index_row ++)
	{
		/* P4 */
		*(ptr_d1V_dy1 + size_col * (index_row - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + size_col)
															  -
															  *(ptr_V + size_col * (index_row - 1) - size_col)
															  ) 
															  / mesh_step_y;
		
		/* P5 */
		for(index_col = 2; index_col <= size_col - 1 ; index_col ++)
		{
		
			*(ptr_d1V_dy1 + size_col * (index_row - 1) + (index_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + (index_col - 1) + size_col)
															  -
															  *(ptr_V + size_col * (index_row - 1) + (index_col - 1) - size_col)
															  ) 
															  / mesh_step_y;
			
		}/* P5*/
		
		/* P6 */
		*(ptr_d1V_dy1 + size_col * (index_row - 1) + (size_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (index_row - 1) + (size_col - 1) + size_col)
															  -
															  *(ptr_V + size_col * (index_row - 1) + (size_col - 1) - size_col)
															  ) 
															  / mesh_step_y;

	}/* P4 - P5 - P6 */
	
	
	
	
	
	/* P7 */
	*(ptr_d1V_dy1 + size_col * (size_row - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + 0)
															  -
															  *(ptr_V + size_col * (size_row - 1) - size_col )
															  ) 
															  / mesh_step_y;


	/* P8 */
	for(index_col = 2; index_col <= size_col - 1; index_col ++)
	{
		*(ptr_d1V_dy1 + size_col * (size_row - 1) + (index_col - 1)) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + (index_col - 1) + 0)
															  -
															  *(ptr_V + size_col * (size_row - 1) + (index_col - 1) - size_col)
															  ) 
															  / mesh_step_y;

	}


	/* P9 */
	*(ptr_d1V_dy1 + size_col * (size_row - 1) + size_col - 1) =  0.5 *  (
															  *(ptr_V + size_col * (size_row - 1) + (size_col - 1) + 0)
															  -
															  *(ptr_V + size_col * (size_row - 1) + (size_col - 1) - size_col)
															  ) 
															  / mesh_step_y;

	
	/* data processed succcessfully */
	return 0;
	
}/* lbmn_mesh_xy_D_calculate_tab_d1V_dy1()*/


/* ================================================= */

int
lbmn_mesh_xy_D_calculate_tab_E1(
            double*     	ptr_E1,
			double*			ptr_E_x,
			double*			ptr_E_y,
			unsigned int    size_row,
            unsigned int    size_col)
{
	/* declaration of variables */

	unsigned int index = 0;
	
	
	/* instructions */
	
	/* check correctness of input parameters */
	
	if(ptr_E1 == NULL)
	{
		return -1;
	}
	
	if(ptr_E_x == NULL)
	{
		return -2;
	}
	
	if(ptr_E_y == NULL)
	{
		return -2;
	}
	
	if(size_row < 3)
	{
		return -3;
	}
	
	if(size_col < 3)
	{
		return -4;
	}
	
	for(index = 0; index < size_row * size_col; index ++)
	{
		*(ptr_E1 + index) = sqrt(
					*(ptr_E_x + index) * *(ptr_E_x + index) 
					+
					*(ptr_E_y + index) * *(ptr_E_y + index)
				 );
	}
	
	return 0;
	
}/* lbmn_mesh_xy_D_calculate_tab_E1 */


/* ================================================== */


int
lbmn_mesh_xy_D_calculate_tab_E2(
            double*     	ptr_E2,
			double*			ptr_E_x,
			double*			ptr_E_y,
			unsigned int    size_row,
            unsigned int    size_col)
{
	/* declaration of variables */

	unsigned int index = 0;
	
	
	/* instructions */
	
	/* check correctness of input parameters */
	
	if(ptr_E2 == NULL)
	{
		return -1;
	}
	
	if(ptr_E_x == NULL)
	{
		return -2;
	}
	
	if(ptr_E_y == NULL)
	{
		return -2;
	}
	
	if(size_row < 3)
	{
		return -3;
	}
	
	if(size_col < 3)
	{
		return -4;
	}
	
	for(index = 0; index < size_row * size_col; index ++)
	{
		*(ptr_E2 + index) =	*(ptr_E_x + index) * *(ptr_E_x + index) 
							+
							*(ptr_E_y + index) * *(ptr_E_y + index);
				 
	}
	
	return 0;
	
}/* lbmn_mesh_xy_D_calculate_tab_E2 */


/* ==== end of file lbmn_mesh_xy_D.c ==== */
