/* lbmn_mesh_xy.h
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef LBMN_MESH_XY_D_H_
#define LBMN_MESH_XY_D_H_

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "lbmn_colormap.h"

/*
* try to allocate problem XY
*/
int
lbmn_mesh_xy_problem_allocate(
			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b,
			
            unsigned int size_row,/* input */
            unsigned int size_col,
            double x_start,
            double x_stop,
            double y_start,
            double y_stop);

/*==========================================================*/

/*
* try to deallocate problem XY
*/
int
lbmn_mesh_xy_problem_destroy(
 			unsigned int* ptr_size_row,/*output*/
            unsigned int* ptr_size_col,
            double* ptr_x_start,
            double* ptr_x_stop,
            double* ptr_y_start,
            double* ptr_y_stop,
			
			double** ptrptr_V,
			unsigned int** ptrptr_Bnd,
			double** ptrptr_E_x,
			double** ptrptr_E_y,
			double** ptrptr_E_1,
			double** ptrptr_E_2,
            unsigned int** ptrptr_isolines_V,
            unsigned int** ptrptr_isolines_E_x,
            unsigned int** ptrptr_isolines_E_y,
            unsigned int** ptrptr_isolines_E_1,
            unsigned int** ptrptr_isolines_E_2,
			unsigned char** ptrptr_workbench_colours_r,
			unsigned char** ptrptr_workbench_colours_g,
			unsigned char** ptrptr_workbench_colours_b,
			unsigned char** ptrptr_workbench_bw_colours_r,
			unsigned char** ptrptr_workbench_bw_colours_g,
			unsigned char** ptrptr_workbench_bw_colours_b);
			
/*
* function used to set mesh points
* "set the voltage and boundraty kind of node of given (i_row, i_col) indices "
*/

int
lbmn_mesh_xy_D_set_point_ij(
                    double* 		ptr_V,
                    unsigned int* 	ptr_Bnd,
                    unsigned int   	size_row,/* size of mesh */
                    unsigned int   	size_col,
                    unsigned int   	i_row,/* point position (1,2, .. , n_row)*/
                    unsigned int   	i_col,/* 1,2,..,n_col*/
                    double 			V , /*input point values*/
					unsigned int    boundary);


/*
* function used to write an rectangle with given colour
*/

int
lbmn_mesh_xy_D_set_rectangle_ij(
                    double*			ptr_V,
                    unsigned int*  ptr_Bnd,
                    unsigned int    size_row,/* size of mesh */
                    unsigned int    size_col,
                    unsigned int    i_row_start,/* rectangle parameters */
					unsigned int    i_row_stop,
					unsigned int    i_col_start,
                    unsigned int    i_col_stop,
                    double			V, /*input rectangle values*/
					unsigned int    boundary);


/* 
* function used to calibrate colormap
* for example, if we know treshold values of V in the entire V mesh,
* we can then transform these V values to colors by any color-mapping algorithm
*/

int
lbmn_mesh_xy_D_find_V_min_max(
                  double*       ptr_min,
                  double*       ptr_max,
                  double*       ptr_V,  
                  unsigned int  size_row,/* size of mesh */
                  unsigned int  size_col);


/*
* functions to "show mesh" on display 
*/

int
lbmn_mesh_xy_D_printf_V(
            double*        ptr_V,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col);

int
lbmn_mesh_xy_D_printf_Bnd(
            unsigned int*  ptr_Bnd,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col);


/*
* functions to write mesh values to output: 
* ".dat" - text files
* ".ppm" - graphical netpbm PBM(P3) (Portable Bitmap Format) files 
*/


/*
* write mesh (type double) to file ppm with given colour mapping function fptr_colormap
* it works with meshes: V, E_x, E_y, E1, E2
*/
int
lbmn_mesh_xy_D_fprintf_mesh_double_netpbm_P3(
            const char*    filename,
            double*        ptr_mesh,
			unsigned int*  ptr_Bnd,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col,
            int(*fptr_colormap)(   
                unsigned char* ptr_red,
                unsigned char* ptr_green,
                unsigned char* ptr_blue,
                double val,
                double val_min,
                double val_max),
			double	V_min,/* range of displayed value */
			double	V_max,
			unsigned char colour_below_red,/* colour - below range */
			unsigned char colour_below_green,
			unsigned char colour_below_blue,
			unsigned char colour_abowe_red,/* colour - abowe range */
			unsigned char colour_abowe_green,
			unsigned char colour_abowe_blue,
			unsigned char* ptr_electrode_colours_red,
			unsigned char* ptr_electrode_colours_green,
			unsigned char* ptr_electrode_colours_blue,
			unsigned int number_of_colours);


/*
* write mesh (type double) to .dat file.
* it works with meshes: V, E_x, E_y, E1, E2
*/
int
lbmn_mesh_xy_D_fprintf_mesh_double_dat(
            const char* 	filename,
            double*        	ptr_mesh,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col);



/*
* write mesh (type unsigned int) to .dat file
*/
int
lbmn_mesh_xy_D_fprintf_mesh_uint_dat(
            const char*     filename,
            unsigned int*   ptr_mesh,
            unsigned int    size_row,/* size of mesh */
            unsigned int    size_col);


/*
* create workbench image 3 r/g/b channels for PPM P3  file (black / white)
*/
int
lbmn_mesh_xy_D_create_image_workbench_bw(
            unsigned char* 	ptr_red,/* targets */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd);/* mesh of boundaries */
			
/*
* create workbench image 3 r/g/b channels for PPM P3  file
*/
int
lbmn_mesh_xy_D_create_image_workbench_colours(
            unsigned char* 	ptr_red,/* targets */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd,/* mesh of boundaries*/
			unsigned char*	ptr_electrode_colours_red,/* colours of each electrode (Bnd = 1,2,3..) (r/g/b) */
			unsigned char*	ptr_electrode_colours_green,
			unsigned char*	ptr_electrode_colours_blue,
			unsigned int	size_electrode_colours);

/*
* write PPM P3 image to file ppm with given colour mapping function fptr_colormap
* it works with meshes: V, E_x, E_y, E1, E2
*/

int
lbmn_mesh_xy_D_fprintf_image_workbench_netpbm_P3(
            const char*    filename,
			unsigned char* 	ptr_red,/* 3 input r/g/b channels */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,  
            unsigned int   size_row,/* size of these r/g/b channels */
            unsigned int   size_col);
			
			

/*
* functions to calculate single and double derivatives of potential V
* d1V_dx1 and d1V_dy1 tables are analogous to V mesh (indices for example)
*/
			
int
lbmn_mesh_xy_D_calculate_tab_d1V_dx1(
            double*     	ptr_d1V_dx1,
			double*			ptr_V,
			unsigned int    size_row,
            unsigned int    size_col,
			double 			mesh_step_x);
			
int
lbmn_mesh_xy_D_calculate_tab_d1V_dy1(
            double*     	ptr_d1V_dy1,
			double*			ptr_V,
			unsigned int    size_row,
            unsigned int    size_col,
			double 			mesh_step_y);

/* 
* functions to calculate electric force (E1 === E === E^1) and its square (E2 === E^2)
* E1 and E2 mesh is analogous to V mesh (indices for example)
*/

int
lbmn_mesh_xy_D_calculate_tab_E1(
            double*     	ptr_E1,
			double*			ptr_E_x,
			double*			ptr_E_y,
			unsigned int    size_row,
            unsigned int    size_col);

int
lbmn_mesh_xy_D_calculate_tab_E2(
            double*     	ptr_E2,
			double*			ptr_E_x,
			double*			ptr_E_y,
			unsigned int    size_row,
            unsigned int    size_col);




#endif /* LBMN_MESH_XY_D_H_ */

/* ==== end of file lbmn_mesh_xy_D.h ==== */
