/* lbmn_relax_xy_D.h
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 

#ifndef LBMN_RELAX_XY_D_H_
#define LBMN_RELAX_XY_D_H_

#include "stdio.h"

/*
* relax_xy_ functions
*
* single relaxation procedures of the whole mesh.
*
* The relaxation schemes have been explained in Liebmann technical documentation:
* 
*   Laplace equation 2D (XY)
*   (Cartesian coordinates)
*   relaxation scheme explained
*   (5 - point star)
*   author: Marcin Kulbaka
*
*/

/*
* "obsolete" "default" relaxation function
*/
double
lbmn_relax_xy_D_iter(
             double*        ptr_V,
             unsigned int*  ptr_Bnd,
             unsigned int   size_row,/* size of mesh */
             unsigned int   size_col);

/*
*	mnemonics of mesh traversing directions:
*
* 	mesh procesing direction "vertically" 
*	B - bottom ->up
*	T - top->down
*	
*	mesh processin "horizontally"
*	L - right -> left
*	R - left  -> right
*/
	 
double
lbmn_relax_xy_D_iter_BL(
                double*         ptr_V,
                unsigned int*   ptr_Bnd,
                unsigned int    size_row,/* size of mesh */
                unsigned int    size_col);
		   
double
lbmn_relax_xy_D_iter_BR(
                double*         ptr_V,
                unsigned int*   ptr_Bnd,
                unsigned int    size_row,/* size of mesh */
                unsigned int    size_col);

double
lbmn_relax_xy_D_iter_TL(
                double*         ptr_V,
                unsigned int*   ptr_Bnd,
                unsigned int    size_row,/* size of mesh */
                unsigned int    size_col);

double
lbmn_relax_xy_D_iter_TR(
                double*         ptr_V,
                unsigned int*   ptr_Bnd,
                unsigned int    size_row,/* size of mesh */
                unsigned int    size_col);


#endif /* LBMN_RELAX_XY_D_H_ */

/* ==== end of file lbmn_relax_xy_D.h ==== */