/* lbmn_relax_zr_D.h
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef LBMN_RALAX_ZR_D_H_
#define LBMN_RELAX_ZR_D_H_

#include "stdio.h"
#include "lbmn_colormap.h"

#include <stdio.h>


/* ================================================================= */

double
lbmn_relax_zr_D_iter(
        double*         ptr_V,
        unsigned int*   ptr_Bnd,
        unsigned int    n_row,/* size of mesh */
        unsigned int    n_col);
   

/*
WARNING - this procedure (BL) is not properly organized (wrong point processing order).
Now it realizes just another BR processing (P1..P9 instead of P3..P7 order).
*/
double
lbmn_relax_zr_D_iter_BL(
        double*         ptr_V,
        unsigned int*   ptr_Bnd,
        unsigned int    n_row,/* size of mesh */
        unsigned int    n_col);

double
lbmn_relax_zr_D_iter_BR(
        double*         ptr_V,
        unsigned int*   ptr_Bnd,
        unsigned int    n_row,/* size of mesh */
        unsigned int    n_col);


int
lbmn_relax_zr_D_set_point(
        double*        ptr_V,
        unsigned int*  ptr_Bnd,
        unsigned int   size_row,/* size of mesh */
        unsigned int   size_col,
        unsigned int   i_row,/* node position (1,2, .. , n_row)*/
        unsigned int   i_col,/* 1,2,..,n_col*/
        double         voltage, /*input node values*/
        unsigned int   boundary);       

int
lbmn_relax_zr_D_find_V_min_max(
        double*         ptr_min,
        double*         ptr_max,
        double*         ptr_V,  
        unsigned int    size_row,/* size of mesh */
        unsigned int    size_col);

int
lbmn_relax_zr_D_printf_V_zr(
        double*        ptr_V,
        unsigned int   size_row,/* size of mesh */
        unsigned int   size_col);

int
lbmn_relax_zr_D_printf_Bnd_zr(
        unsigned int*  ptr_Bnd,
        unsigned int   size_row,/* size of mesh */
        unsigned int   size_col);


int
lbmn_relax_zr_D_fprintf_V_netpbm_P3(
            const char*    filename,
            double*        ptr_V,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col,
            int(*fptr_colormap)(   
                unsigned char* ptr_red,
                unsigned char* ptr_green,
                unsigned char* ptr_blue,
                double val,
                double val_min,
                double val_max));

int
lbmn_relax_zr_D_fprintf_any_workbench_netpbm_P3(
            const char*    	filename,
            unsigned char* 	ptr_red,
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col);

int
lbmn_relax_zr_D_fprintf_any_V_z(
            const char* filename,
            double* ptr_z,
            double* ptr_V,
            unsigned int size_col);/* size of mesh (only columns)*/

int
lbmn_relax_zr_D_fprintf_V_dat(
            const char* filename,
            double*        ptr_V,
            unsigned int   size_row,/* size of mesh */
            unsigned int   size_col);

int
lbmn_relax_zr_D_fprintf_Bnd_dat(
            const char*     filename,
            unsigned int*   ptr_Bnd,
            unsigned int    size_row,/* size of mesh */
            unsigned int    size_col);


int
lbmn_relax_zr_D_calculate_all_tab_V_z_h(
            double*     	ptr_V_z,
			double*			ptr_d1V_dz1_h,
			double*			ptr_d2V_dz2_h,
            double*     	ptr_V,
            unsigned int    size_col,
			double 			mesh_step_z);

int
lbmn_relax_zr_D_calculate_tab_V_z_2h(
            double*			ptr_d1V_dz1_2h,
			double*			ptr_d2V_dz2_2h,
            double*     	ptr_V,
            unsigned int    size_col,
			double 			mesh_step_z);			
			
int
lbmn_relax_zr_D_create_image_workbench_bw(
            unsigned char* 	ptr_red,/* targets */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd,/* mesh of boundaries*/
			unsigned char	axis_colour_red,
			unsigned char	axis_colour_green,
			unsigned char  	axis_colour_blue);
			
		
int
lbmn_relax_zr_D_create_image_workbench_colours(
            unsigned char* 	ptr_red,/* targets */
			unsigned char*	ptr_green,
			unsigned char*	ptr_blue,
            unsigned int   	size_row,/* size of mesh */
            unsigned int   	size_col,
			unsigned int*	ptr_Bnd,/* mesh of boundaries*/
			unsigned char*	ptr_electrode_colours_red,/* colours of each electrode (Bnd = 1,2,3..) (r/g/b) */
			unsigned char*	ptr_electrode_colours_green,
			unsigned char*	ptr_electrode_colours_blue,
			unsigned int	size_electrode_colours,
			unsigned char	axis_colour_red,
			unsigned char	axis_colour_green,
			unsigned char  	axis_colour_blue);
			
#endif /* LBMN_RELAX_ZR_D_H_ */

/* ==== end of file lbmn_relax_zr_D.h ==== */