/* demo_xy_ptr_ptr.c
 * 
 * Copyright (C) 2024 Marcin Kulbaka
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>


int
fun_allocate_V(
	double** ptrptr_V,
	unsigned int* ptr_size,
	unsigned int size
);

int
fun_destroy_V(
	double** ptrptr_V,
	unsigned int* ptr_size
);

/****************** main program *********************/

int
main(void)
{
	
	/* variables */
	int i = 0;
	int ret_val = 0;
	
	double* ptr_V = NULL;
	unsigned int size_V = 0;
	
	/* problem size */
	unsigned int size =5;
	
	
	/* instructions */
	
	printf("\n*** PROGRAM DEMO_XY_PTR_PTR ***\n\n");

	printf("Allocation of memory block ptr_V\n");
	
	ret_val = fun_allocate_V(&ptr_V, &size_V, size);

	printf("ret_val == %i\n", ret_val);
	
	if(ret_val != 0)
	{
		printf("FAILURE - problem with memory allocation\n");
	}
	else
	{/* OK */

		printf("SUCCESS - memory allocated successfully\n");
		
		printf("ptr_V  == %p\n", (void*) ptr_V);
		printf("size_V == %u\n", size_V);
			
		/* ACTION */
		
		printf("filling memory block...\n");
		*(ptr_V + 0) = 0.5;
		*(ptr_V + 1) = 1.5;
		*(ptr_V + 2) = 2.5;
		*(ptr_V + 3) = 3.5;
		*(ptr_V + 4) = 4.5;
		
		printf("reading ptr_V memory block\n");
		
		for(i = 0; i < 5; i++)
		{
			printf("*(ptr_V + %i) == %f\n",i, *(ptr_V + i) );
		}
		
		
		/* CLEAR */
		printf("free memory...\n");
		
		ret_val = fun_destroy_V(&ptr_V, &size_V);
		
		printf("ret_val == %i\n", ret_val);
		
		if(ret_val != 0)
		{
			printf("FAILURE - problem with memory deallocation\n");
		}
		else
		{/* OK */
	
			printf("memory deallocated successfully\n");
		
			printf("ptr_V  == %p\n", (void*) ptr_V);
			printf("size_V == %u\n", size_V);
		}/* OK */
	
	}/* OK */
	
	printf("\n*** END OF PROGRAM DEMO_XY_PTR_PTR ***\n");
    return 0;
	
}/* main program */

/*=============================================*/

int
fun_allocate_V(
	double** ptrptr_V,
	unsigned int* ptr_size,
	unsigned int size)
{
	*ptrptr_V = (double*)malloc(sizeof(double) * size);
	
	if(*ptrptr_V == NULL)
	{/* error */

		printf("FAILURE - could not allocate memory");
		
		*ptr_size = 0;
		
		return -1;
	}/* error */
	else
	{/* OK */

		*ptr_size = size;
		
		/* return 0 at the end of function */
		
	}/* OK */
	
	return 0;

}/* fun_allocate_V */

/*===========================================*/

int
fun_destroy_V(
	double** ptrptr_V,
	unsigned int* ptr_size)
{
	if(*ptrptr_V == NULL)
	{
		printf("fun_destroy_V - could not free NULL memory block!");
		return -1;
	}
	else
	{
		printf("fun_destroy_V - deallocation of memory...\n");
		
		free(*ptrptr_V);
		*(ptrptr_V) = NULL;

	
		*ptr_size = 0;
		/* return 0 */
	}
	
	return 0;
	
}/* fun_destroy_V */

/* ==== end of file demo_xy_ptr_ptr.c ==== */
